@@ -89,64 +89,112 @@ extern int cm_diskCacheEnabled;
 /* set this to 1 when we are terminating to prevent access attempts */
 static int buf_ShutdownFlag = 0;
 
+void buf_HoldLocked(cm_buf_t *bp)
+{
+    osi_assert(bp->magic == CM_BUF_MAGIC);
+    bp->refCount++;
+}
+
 /* hold a reference to an already held buffer */
 void buf_Hold(cm_buf_t *bp)
 {
-    osi_assert(bp->magic == CM_BUF_MAGIC);
     lock_ObtainWrite(&buf_globalLock);
-    bp->refCount++;
+    buf_HoldLocked(bp);
     lock_ReleaseWrite(&buf_globalLock);
 }
 
-/* incremental sync daemon.  Writes 1/10th of all the buffers every 5000 ms */
+/* code to drop reference count while holding buf_globalLock */
+void buf_ReleaseLocked(cm_buf_t *bp)
+{
+    /* ensure that we're in the LRU queue if our ref count is 0 */
+    osi_assert(bp->magic == CM_BUF_MAGIC);
+#ifdef DEBUG
+    if (bp->refCount == 0)
+	osi_panic("buf refcount 0",__FILE__,__LINE__);;
+#else
+    osi_assert(bp->refCount > 0);
+#endif
+    if (--bp->refCount == 0) {
+        if (!(bp->flags & CM_BUF_INLRU)) {
+            osi_QAdd((osi_queue_t **) &cm_data.buf_freeListp, &bp->q);
+
+            /* watch for transition from empty to one element */
+            if (!cm_data.buf_freeListEndp)
+                cm_data.buf_freeListEndp = cm_data.buf_freeListp;
+            bp->flags |= CM_BUF_INLRU;
+        }
+    }
+}       
+
+/* release a buffer.  Buffer must be referenced, but unlocked. */
+void buf_Release(cm_buf_t *bp)
+{
+    lock_ObtainWrite(&buf_globalLock);
+    buf_ReleaseLocked(bp);
+    lock_ReleaseWrite(&buf_globalLock);
+}
+
+/* incremental sync daemon.  Writes all dirty buffers every 5000 ms */
 void buf_IncrSyncer(long parm)
 {
-    cm_buf_t *bp;			/* buffer we're hacking on; held */
+    cm_buf_t **bpp, *bp;
     long i;				/* counter */
-    long nAtOnce;			/* how many to do at once */
+    long wasDirty = 0;
     cm_req_t req;
 
-    lock_ObtainWrite(&buf_globalLock);
-    bp = cm_data.buf_allp;
-    bp->refCount++;
-    lock_ReleaseWrite(&buf_globalLock);
-    nAtOnce = (long)sqrt((double)cm_data.buf_nbuffers);
     while (buf_ShutdownFlag == 0) {
+	if (!wasDirty) {
 #ifndef DJGPP
-        i = SleepEx(5000, 1);
-        if (i != 0) continue;
+	    i = SleepEx(5000, 1);
+	    if (i != 0) continue;
 #else
-        thrd_Sleep(5000);
+	    thrd_Sleep(5000);
 #endif /* DJGPP */
-            
-        if (buf_ShutdownFlag == 1)
-            return;
+	}
+
+	wasDirty = 0;
 
         /* now go through our percentage of the buffers */
-        for (i=0; i<nAtOnce; i++) {
-            /* don't want its identity changing while we're
-             * messing with it, so must do all of this with
-             * bp held.
-             */
+        for (bpp = &cm_data.buf_dirtyListp; bp = *bpp; ) {
 
-            /* start cleaning the buffer; don't touch log pages since
-             * the log code counts on knowing exactly who is writing
-             * a log page at any given instant.
-             */
-            cm_InitReq(&req);
-            req.flags |= CM_REQ_NORETRY;
-            buf_CleanAsync(bp, &req);
+	    /* all dirty buffers are held when they are added to the
+	     * dirty list.  No need for an additional hold.
+	     */
 
-            /* now advance to the next buffer; the allp chain never changes,
-             * and so can be followed even when holding no locks.
-             */
-            lock_ObtainWrite(&buf_globalLock);
-            buf_LockedRelease(bp);
-            bp = bp->allp;
-            if (!bp) 
-                bp = cm_data.buf_allp;
-            bp->refCount++;
-            lock_ReleaseWrite(&buf_globalLock);
+	    if (bp->flags & CM_BUF_DIRTY) {
+		/* start cleaning the buffer; don't touch log pages since
+ 		 * the log code counts on knowing exactly who is writing
+		 * a log page at any given instant.
+		 */
+		cm_InitReq(&req);
+		req.flags |= CM_REQ_NORETRY;
+		wasDirty |= buf_CleanAsync(bp, &req);
+	    }
+
+	    /* the buffer may or may not have been dirty
+	     * and if dirty may or may not have been cleaned
+	     * successfully.  check the dirty flag again.  
+	     */
+	    if (!(bp->flags & CM_BUF_DIRTY)) {
+		lock_ObtainMutex(&bp->mx);
+		if (!(bp->flags & CM_BUF_DIRTY)) {
+		    /* remove the buffer from the dirty list */
+		    lock_ObtainWrite(&buf_globalLock);
+		    *bpp = bp->dirtyp;
+		    bp->dirtyp = NULL;
+		    if (cm_data.buf_dirtyListp == NULL)
+			cm_data.buf_dirtyListEndp = NULL;
+		    buf_ReleaseLocked(bp);
+		    lock_ReleaseWrite(&buf_globalLock);
+		} else {
+		    /* advance the pointer so we don't loop forever */
+		    bpp = &bp->dirtyp;
+		}
+		lock_ReleaseMutex(&bp->mx);
+	    } else {
+		/* advance the pointer so we don't loop forever */
+		bpp = &bp->dirtyp;
+	    }
         }	/* for loop over a bunch of buffers */
     }		/* whole daemon's while loop */
 }
@@ -426,19 +474,13 @@ long buf_SetNBuffers(afs_uint64 nbuffers)
         return CM_ERROR_INVAL;
 }
 
-/* release a buffer.  Buffer must be referenced, but unlocked. */
-void buf_Release(cm_buf_t *bp)
-{
-    lock_ObtainWrite(&buf_globalLock);
-    buf_LockedRelease(bp);
-    lock_ReleaseWrite(&buf_globalLock);
-}
-
 /* wait for reading or writing to clear; called with write-locked
  * buffer and unlocked scp and returns with locked buffer.
  */
 void buf_WaitIO(cm_scache_t * scp, cm_buf_t *bp)
 {
+    int release = 0;
+
     if (scp)
         osi_assert(scp->magic == CM_SCACHE_MAGIC);
     osi_assert(bp->magic == CM_BUF_MAGIC);
@@ -455,29 +497,33 @@ void buf_WaitIO(cm_scache_t * scp, cm_buf_t *bp)
         if ( bp->flags & CM_BUF_WAITING ) {
             bp->waitCount++;
             bp->waitRequests++;
-            osi_Log1(afsd_logp, "buf_WaitIO CM_BUF_WAITING already set for 0x%p", bp);
+            osi_Log1(buf_logp, "buf_WaitIO CM_BUF_WAITING already set for 0x%p", bp);
         } else {
-            osi_Log1(afsd_logp, "buf_WaitIO CM_BUF_WAITING set for 0x%p", bp);
+            osi_Log1(buf_logp, "buf_WaitIO CM_BUF_WAITING set for 0x%p", bp);
             bp->flags |= CM_BUF_WAITING;
             bp->waitCount = bp->waitRequests = 1;
         }
         osi_SleepM((LONG_PTR)bp, &bp->mx);
+
+	smb_UpdateServerPriority();
+
         lock_ObtainMutex(&bp->mx);
-        osi_Log1(afsd_logp, "buf_WaitIO conflict wait done for 0x%p", bp);
+        osi_Log1(buf_logp, "buf_WaitIO conflict wait done for 0x%p", bp);
         bp->waitCount--;
         if (bp->waitCount == 0) {
-            osi_Log1(afsd_logp, "buf_WaitIO CM_BUF_WAITING reset for 0x%p", bp);
+            osi_Log1(buf_logp, "buf_WaitIO CM_BUF_WAITING reset for 0x%p", bp);
             bp->flags &= ~CM_BUF_WAITING;
             bp->waitRequests = 0;
         }
 
         if ( !scp ) {
-            scp = cm_FindSCache(&bp->fid);
+            if (scp = cm_FindSCache(&bp->fid))
+		 release = 1;
         }
         if ( scp ) {
             lock_ObtainMutex(&scp->mx);
             if (scp->flags & CM_SCACHEFLAG_WAITING) {
-                osi_Log1(afsd_logp, "buf_WaitIO waking scp 0x%p", scp);
+                osi_Log1(buf_logp, "buf_WaitIO waking scp 0x%p", scp);
                 osi_Wakeup((LONG_PTR)&scp->flags);
             }
 	    lock_ReleaseMutex(&scp->mx);
@@ -488,33 +534,19 @@ void buf_WaitIO(cm_scache_t * scp, cm_buf_t *bp)
      * the I/O to complete.  Do so.
      */
     if (bp->flags & CM_BUF_WAITING) {
-        osi_Log1(afsd_logp, "buf_WaitIO Waking bp 0x%p", bp);
+        osi_Log1(buf_logp, "buf_WaitIO Waking bp 0x%p", bp);
         osi_Wakeup((LONG_PTR) bp);
     }
-    osi_Log1(afsd_logp, "WaitIO finished wait for bp 0x%p", bp);
+    osi_Log1(buf_logp, "WaitIO finished wait for bp 0x%p", bp);
+
+    if (scp && release)
+	cm_ReleaseSCache(scp);
 }
 
-/* code to drop reference count while holding buf_globalLock */
-void buf_LockedRelease(cm_buf_t *bp)
-{
-    /* ensure that we're in the LRU queue if our ref count is 0 */
-    osi_assert(bp->refCount > 0);
-    if (--bp->refCount == 0) {
-        if (!(bp->flags & CM_BUF_INLRU)) {
-            osi_QAdd((osi_queue_t **) &cm_data.buf_freeListp, &bp->q);
-
-            /* watch for transition from empty to one element */
-            if (!cm_data.buf_freeListEndp)
-                cm_data.buf_freeListEndp = cm_data.buf_freeListp;
-            bp->flags |= CM_BUF_INLRU;
-        }
-    }
-}       
-
 /* find a buffer, if any, for a particular file ID and offset.  Assumes
  * that buf_globalLock is write locked when called.
  */
-cm_buf_t *buf_LockedFind(struct cm_scache *scp, osi_hyper_t *offsetp)
+cm_buf_t *buf_FindLocked(struct cm_scache *scp, osi_hyper_t *offsetp)
 {
     long i;
     cm_buf_t *bp;
@@ -524,7 +556,7 @@ cm_buf_t *buf_LockedFind(struct cm_scache *scp, osi_hyper_t *offsetp)
         if (cm_FidCmp(&scp->fid, &bp->fid) == 0
              && offsetp->LowPart == bp->offset.LowPart
              && offsetp->HighPart == bp->offset.HighPart) {
-            bp->refCount++;
+            buf_HoldLocked(bp);
             break;
         }
     }
@@ -541,7 +573,7 @@ cm_buf_t *buf_Find(struct cm_scache *scp, osi_hyper_t *offsetp)
     cm_buf_t *bp;
 
     lock_ObtainWrite(&buf_globalLock);
-    bp = buf_LockedFind(scp, offsetp);
+    bp = buf_FindLocked(scp, offsetp);
     lock_ReleaseWrite(&buf_globalLock);
 
     return bp;
@@ -553,33 +585,68 @@ cm_buf_t *buf_Find(struct cm_scache *scp, osi_hyper_t *offsetp)
  * Makes sure that there's only one person writing this block
  * at any given time, and also ensures that the log is forced sufficiently far,
  * if this buffer contains logged data.
+ *
+ * Returns non-zero if the buffer was dirty.
  */
-void buf_LockedCleanAsync(cm_buf_t *bp, cm_req_t *reqp)
+long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
 {
     long code = 0;
+    long isdirty = 0;
+    cm_scache_t * scp = NULL;
 
     osi_assert(bp->magic == CM_BUF_MAGIC);
 
     while ((bp->flags & CM_BUF_DIRTY) == CM_BUF_DIRTY) {
+	isdirty = 1;
         lock_ReleaseMutex(&bp->mx);
 
-	osi_Log1(afsd_logp, "buf_LockedCleanAsync starts I/O on 0x%p", bp);
-        code = (*cm_buf_opsp->Writep)(&bp->fid, &bp->offset,
-                                       cm_data.buf_blockSize, 0, bp->userp,
-                                       reqp);
-	osi_Log2(afsd_logp, "buf_LockedCleanAsync I/O on 0x%p, done=%d", bp, code);
-                
-        lock_ObtainMutex(&bp->mx);
-        if (code) 
-            break;
+	scp = cm_FindSCache(&bp->fid);
+	if (scp) {
+	    osi_Log2(buf_logp, "buf_CleanAsyncLocked starts I/O on scp 0x%p buf 0x%p", scp, bp);
+	    code = (*cm_buf_opsp->Writep)(scp, &bp->offset,
+					   cm_data.buf_blockSize, 0, bp->userp,
+					   reqp);
+	    osi_Log3(buf_logp, "buf_CleanAsyncLocked I/O on scp 0x%p buf 0x%p, done=%d", scp, bp, code);
+
+	    cm_ReleaseSCache(scp);
+	    scp = NULL;
+	} else {
+	    osi_Log1(buf_logp, "buf_CleanAsyncLocked unable to start I/O - scp not found buf 0x%p", bp);
+	    code = CM_ERROR_NOSUCHFILE;
+	}    
+        
+	lock_ObtainMutex(&bp->mx);
+	/* if the Write routine returns No Such File, clear the dirty flag
+	 * because we aren't going to be able to write this data to the file
+	 * server.
+	 */
+	if (code == CM_ERROR_NOSUCHFILE){
+	    bp->flags &= ~CM_BUF_DIRTY;
+	    bp->flags |= CM_BUF_ERROR;
+	    bp->error = CM_ERROR_NOSUCHFILE;
+	    bp->dataVersion = -1; /* bad */
+	    bp->dirtyCounter++;
+	}
 
 #ifdef DISKCACHE95
         /* Disk cache support */
         /* write buffer to disk cache (synchronous for now) */
         diskcache_Update(bp->dcp, bp->datap, cm_data.buf_blockSize, bp->dataVersion);
 #endif /* DISKCACHE95 */
+
+	/* if we get here and retries are not permitted 
+	 * then we need to exit this loop regardless of 
+	 * whether or not we were able to clear the dirty bit
+	 */
+	if (reqp->flags & CM_REQ_NORETRY)
+	    break;
     };
 
+    if (!(bp->flags & CM_BUF_DIRTY)) {
+	/* remove buffer from dirty buffer queue */
+
+    }
+
     /* do logging after call to GetLastError, or else */
         
     /* if someone was waiting for the I/O that just completed or failed,
@@ -590,6 +657,7 @@ void buf_LockedCleanAsync(cm_buf_t *bp, cm_req_t *reqp)
         osi_Log1(buf_logp, "buf_WaitIO Waking bp 0x%p", bp);
         osi_Wakeup((LONG_PTR) bp);
     }
+    return isdirty;
 }
 
 /* Called with a zero-ref count buffer and with the buf_globalLock write locked.
@@ -687,7 +755,11 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
         lock_ObtainWrite(&buf_globalLock);
         /* check to see if we lost the race */
         if (scp) {
-            if (bp = buf_LockedFind(scp, offsetp)) {
+            if (bp = buf_FindLocked(scp, offsetp)) {
+		/* Do not call buf_ReleaseLocked() because we 
+		 * do not want to allow the buffer to be added
+		 * to the free list.
+		 */
                 bp->refCount--;
                 lock_ReleaseWrite(&buf_globalLock);
                 return CM_BUF_EXISTS;
@@ -751,7 +823,7 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
                  * just the lock required to minimize contention
                  * on the big lock.
                  */
-                bp->refCount++;
+                buf_HoldLocked(bp);
                 lock_ReleaseWrite(&buf_globalLock);
 
                 /* grab required lock and clean; this only
@@ -786,6 +858,9 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
             if (scp) {
                 bp->flags |= CM_BUF_INHASH;
                 bp->fid = scp->fid;
+#ifdef DEBUG
+		bp->scp = scp;
+#endif
                 bp->offset = *offsetp;
                 i = BUF_HASH(&scp->fid, offsetp);
                 bp->hashp = cm_data.buf_hashTablepp[i];
@@ -833,6 +908,7 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
             return 0;
         } /* for all buffers in lru queue */
         lock_ReleaseWrite(&buf_globalLock);
+		Sleep(100);		/* give some time for a buffer to be freed */
     }	/* while loop over everything */
     /* not reached */
 } /* the proc */
@@ -885,7 +961,7 @@ long buf_GetNew(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bufpp)
      */
     lock_ReleaseMutex(&bp->mx);
     *bufpp = bp;
-    osi_Log3(buf_logp, "buf_GetNew returning bp 0x%p for file 0x%p, offset 0x%x",
+    osi_Log3(buf_logp, "buf_GetNew returning bp 0x%p for scp 0x%p, offset 0x%x",
               bp, scp, offsetp->LowPart);
     return 0;
 }
@@ -917,16 +993,17 @@ long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bufpp)
         if (bp) {
             /* lock it and break out */
             lock_ObtainMutex(&bp->mx);
-            break;
 
 #ifdef DISKCACHE95
             /* touch disk chunk to update LRU info */
             diskcache_Touch(bp->dcp);
 #endif /* DISKCACHE95 */
+            break;
         }
 
         /* otherwise, we have to create a page */
         code = buf_GetNewLocked(scp, &pageOffset, &bp);
+	/* bp->mx is now held */
 
         /* check if the buffer was created in a race condition branch.
          * If so, go around so we can hold a reference to it. 
@@ -1032,7 +1109,7 @@ long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bufpp)
     }
     lock_ReleaseWrite(&buf_globalLock);
 
-    osi_Log3(buf_logp, "buf_Get returning bp 0x%p for file 0x%p, offset 0x%x",
+    osi_Log3(buf_logp, "buf_Get returning bp 0x%p for scp 0x%p, offset 0x%x",
               bp, scp, offsetp->LowPart);
 #ifdef TESTING
     buf_ValidateBufQueues();
@@ -1065,13 +1142,16 @@ long buf_CountFreeList(void)
 }
 
 /* clean a buffer synchronously */
-void buf_CleanAsync(cm_buf_t *bp, cm_req_t *reqp)
+long buf_CleanAsync(cm_buf_t *bp, cm_req_t *reqp)
 {
+    long code;
     osi_assert(bp->magic == CM_BUF_MAGIC);
 
     lock_ObtainMutex(&bp->mx);
-    buf_LockedCleanAsync(bp, reqp);
+    code = buf_CleanAsyncLocked(bp, reqp);
     lock_ReleaseMutex(&bp->mx);
+
+    return code;
 }       
 
 /* wait for a buffer's cleaning to finish */
@@ -1096,13 +1176,39 @@ void buf_SetDirty(cm_buf_t *bp)
     osi_assert(bp->magic == CM_BUF_MAGIC);
     osi_assert(bp->refCount > 0);
 	
-    osi_Log1(buf_logp, "buf_SetDirty 0x%p", bp);
-
+    if (bp->flags & CM_BUF_DIRTY) {
+	osi_Log1(buf_logp, "buf_SetDirty 0x%p already dirty", bp);
+    } else {
+	osi_Log1(buf_logp, "buf_SetDirty 0x%p", bp);
+    }
     /* set dirty bit */
     bp->flags |= CM_BUF_DIRTY;
 
     /* and turn off EOF flag, since it has associated data now */
     bp->flags &= ~CM_BUF_EOF;
+
+    /* and add to the dirty list.  
+     * we obtain a hold on the buffer for as long as it remains 
+     * in the list.  buffers are only removed from the list by 
+     * the buf_IncrSyncer function regardless of when else the
+     * dirty flag might be cleared.
+     *
+     * This should never happen but just in case there is a bug
+     * elsewhere, never add to the dirty list if the buffer is 
+     * already there.
+     */
+    lock_ObtainWrite(&buf_globalLock);
+    if (bp->dirtyp == NULL && cm_data.buf_dirtyListEndp != bp) {
+	buf_HoldLocked(bp);
+	if (!cm_data.buf_dirtyListp) {
+	    cm_data.buf_dirtyListp = cm_data.buf_dirtyListEndp = bp;
+	} else {
+	    cm_data.buf_dirtyListEndp->dirtyp = bp;
+	    cm_data.buf_dirtyListEndp = bp;
+	}
+	bp->dirtyp = NULL;
+    }
+    lock_ReleaseWrite(&buf_globalLock);
 }
 
 /* clean all buffers, reset log pointers and invalidate all buffers.
@@ -1135,17 +1241,19 @@ long buf_CleanAndReset(void)
     for(i=0; i<cm_data.buf_hashSize; i++) {
         for(bp = cm_data.buf_hashTablepp[i]; bp; bp = bp->hashp) {
             if ((bp->flags & CM_BUF_DIRTY) == CM_BUF_DIRTY) {
-                bp->refCount++;
+                buf_HoldLocked(bp);
                 lock_ReleaseWrite(&buf_globalLock);
 
                 /* now no locks are held; clean buffer and go on */
                 cm_InitReq(&req);
-                buf_CleanAsync(bp, &req);
-                buf_CleanWait(NULL, bp);
+		req.flags |= CM_REQ_NORETRY;
+
+		buf_CleanAsync(bp, &req);
+		buf_CleanWait(NULL, bp);
 
                 /* relock and release buffer */
                 lock_ObtainWrite(&buf_globalLock);
-                buf_LockedRelease(bp);
+                buf_ReleaseLocked(bp);
             } /* dirty */
         } /* over one bucket */
     }	/* for loop over all hash buckets */
@@ -1225,7 +1333,6 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
     osi_hyper_t bufEnd;
     long code;
     long bufferPos;
-    int didRelease;
     long i;
 
     /* assert that cm_bufCreateLock is held in write mode */
@@ -1240,10 +1347,9 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
         return 0;
     }
 
-    bufp->refCount++;
+    buf_HoldLocked(bufp);
     lock_ReleaseWrite(&buf_globalLock);
     for(; bufp; bufp = nbufp) {
-        didRelease = 0;
         lock_ObtainMutex(&bufp->mx);
 
         bufEnd.HighPart = 0;
@@ -1264,14 +1370,16 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
                           | CM_SCACHESYNC_GETSTATUS
                           | CM_SCACHESYNC_SETSIZE
                           | CM_SCACHESYNC_BUFLOCKED);
-        /* if we succeeded in our locking, and this applies to the right
+
+	
+	lock_ObtainWrite(&buf_globalLock);
+	/* if we succeeded in our locking, and this applies to the right
          * file, and the truncate request overlaps the buffer either
          * totally or partially, then do something.
          */
         if (code == 0 && cm_FidCmp(&bufp->fid, &scp->fid) == 0
              && LargeIntegerLessThan(*sizep, bufEnd)) {
 
-            lock_ObtainWrite(&buf_globalLock);
 
             /* destroy the buffer, turning off its dirty bit, if
              * we're truncating the whole buffer.  Otherwise, set
@@ -1298,42 +1406,34 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
                 memset(bufp->datap + bufferPos, 0,
                         cm_data.buf_blockSize - bufferPos);
             }
-
-            lock_ReleaseWrite(&buf_globalLock);
         }
 		
+	cm_SyncOpDone( scp, bufp, 
+		       CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS
+		       | CM_SCACHESYNC_SETSIZE | CM_SCACHESYNC_BUFLOCKED);
+
         lock_ReleaseMutex(&scp->mx);
         lock_ReleaseMutex(&bufp->mx);
-        if (!didRelease) {
-            lock_ObtainWrite(&buf_globalLock);
-            nbufp = bufp->fileHashp;
-            if (nbufp) nbufp->refCount++;
-            buf_LockedRelease(bufp);
-            lock_ReleaseWrite(&buf_globalLock);
-        }
-
-        /* bail out early if we fail */
-        if (code) {
-            /* at this point, nbufp is held; bufp has already been
-             * released.
-             */
-            if (nbufp) 
-                buf_Release(nbufp);
-
-#ifdef TESTING
-            buf_ValidateBufQueues();
-#endif /* TESTING */
-
-            return code;
-        }
+    
+	if (!code) {
+	    nbufp = bufp->fileHashp;
+	    if (nbufp) 
+		buf_HoldLocked(nbufp);
+	} else {
+	    /* This forces the loop to end and the error code
+	     * to be returned. */
+	    nbufp = NULL;
+	}
+	buf_ReleaseLocked(bufp);
+	lock_ReleaseWrite(&buf_globalLock);
     }
 
 #ifdef TESTING
     buf_ValidateBufQueues();
 #endif /* TESTING */
 
-    /* success */
-    return 0;
+    /* done */
+    return code;
 }
 
 long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
@@ -1350,8 +1450,9 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     lock_ObtainWrite(&buf_globalLock);
     bp = cm_data.buf_fileHashTablepp[i];
     if (bp) 
-        bp->refCount++;
+        buf_HoldLocked(bp);
     lock_ReleaseWrite(&buf_globalLock);
+    
     for (; bp; bp = nbp) {
         didRelease = 0;	/* haven't released this buffer yet */
 
@@ -1360,7 +1461,7 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
             lock_ObtainMutex(&bp->mx);
 
             /* start cleaning the buffer, and wait for it to finish */
-            buf_LockedCleanAsync(bp, reqp);
+            buf_CleanAsyncLocked(bp, reqp);
             buf_WaitIO(scp, bp);
             lock_ReleaseMutex(&bp->mx);
 
@@ -1374,10 +1475,10 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
              */
             if (!(bp->flags & CM_BUF_DIRTY)) {
                 if (bp->refCount == 1) {	/* bp is held above */
-                    buf_LockedRelease(bp);
                     nbp = bp->fileHashp;
                     if (nbp) 
-                        nbp->refCount++;
+                        buf_HoldLocked(nbp);
+                    buf_ReleaseLocked(bp);
                     didRelease = 1;
                     buf_Recycle(bp);
                 }
@@ -1390,15 +1491,16 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
       skip:
         if (!didRelease) {
             lock_ObtainWrite(&buf_globalLock);
-            if (nbp = bp->fileHashp) 
-                nbp->refCount++;
-            buf_LockedRelease(bp);
+            nbp = bp->fileHashp;
+	    if (nbp)
+                buf_HoldLocked(nbp);
+            buf_ReleaseLocked(bp);
             lock_ReleaseWrite(&buf_globalLock);
         }
     }	/* for loop over a bunch of buffers */
 
 #ifdef TESTING
-            buf_ValidateBufQueues();
+    buf_ValidateBufQueues();
 #endif /* TESTING */
 
     /* done */
@@ -1407,18 +1509,18 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 
 long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
 {
-    long code;
+    long code = 0;
+    long wasDirty = 0;
     cm_buf_t *bp;		/* buffer we're hacking on */
     cm_buf_t *nbp;		/* next one */
     long i;
 
     i = BUF_FILEHASH(&scp->fid);
 
-    code = 0;
     lock_ObtainWrite(&buf_globalLock);
     bp = cm_data.buf_fileHashTablepp[i];
     if (bp) 
-        bp->refCount++;
+        buf_HoldLocked(bp);
     lock_ReleaseWrite(&buf_globalLock);
     for (; bp; bp = nbp) {
         /* clean buffer synchronously */
@@ -1431,12 +1533,11 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
                 bp->userp = userp;
                 lock_ReleaseMutex(&bp->mx);
             }   
-            buf_CleanAsync(bp, reqp);
-            buf_CleanWait(scp, bp);
+            wasDirty = buf_CleanAsync(bp, reqp);
+	    buf_CleanWait(scp, bp);
             lock_ObtainMutex(&bp->mx);
             if (bp->flags & CM_BUF_ERROR) {
-                if (code == 0 || code == -1) 
-                    code = bp->error;
+		code = bp->error;
                 if (code == 0) 
                     code = -1;
             }
@@ -1444,10 +1545,10 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
         }
 
         lock_ObtainWrite(&buf_globalLock);
-        buf_LockedRelease(bp);
         nbp = bp->fileHashp;
         if (nbp) 
-            nbp->refCount++;
+            buf_HoldLocked(nbp);
+        buf_ReleaseLocked(bp);
         lock_ReleaseWrite(&buf_globalLock);
     }	/* for loop over a bunch of buffers */
 
@@ -1469,31 +1570,31 @@ buf_ValidateBufQueues(void)
     lock_ObtainRead(&buf_globalLock);
     for (bp = cm_data.buf_freeListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
         if (bp->magic != CM_BUF_MAGIC)
-            DebugBreak();
+            osi_panic("buf magic error",__FILE__,__LINE__);
         countb++;
         bpb = bp;
     }
 
     for (bp = cm_data.buf_freeListp; bp; bp=(cm_buf_t *) osi_QNext(&bp->q)) {
         if (bp->magic != CM_BUF_MAGIC)
-            DebugBreak();
+            osi_panic("buf magic error",__FILE__,__LINE__);
         countf++;
         bpf = bp;
     }
 
     for (bp = cm_data.buf_allp; bp; bp=bp->allp) {
         if (bp->magic != CM_BUF_MAGIC)
-            DebugBreak();
+            osi_panic("buf magic error",__FILE__,__LINE__);
         counta++;
         bpa = bp;
     }
     lock_ReleaseRead(&buf_globalLock);
 
     if (countb != countf)
-        DebugBreak();
+	osi_panic("buf magic error",__FILE__,__LINE__);
 
     if (counta != cm_data.buf_nbuffers)
-        DebugBreak();	
+	osi_panic("buf magic error",__FILE__,__LINE__);
 }
 #endif /* TESTING */
 
@@ -1558,3 +1659,44 @@ void buf_ForceTrace(BOOL flush)
         FlushFileBuffers(handle);
     CloseHandle(handle);
 }
+
+long buf_DirtyBuffersExist(cm_fid_t *fidp)
+{
+    cm_buf_t *bp;
+    afs_uint32 bcount = 0;
+
+    for (bp = cm_data.buf_allp; bp; bp=bp->allp, bcount++) {
+	if (!cm_FidCmp(fidp, &bp->fid) && (bp->flags & CM_BUF_DIRTY))
+	    return 1;
+    }
+    return 0;
+}
+
+#if 0
+long buf_CleanDirtyBuffers(cm_scache_t *scp)
+{
+    cm_buf_t *bp;
+    afs_uint32 bcount = 0;
+    cm_fid_t * fidp = &scp->fid;
+
+    for (bp = cm_data.buf_allp; bp; bp=bp->allp, bcount++) {
+	if (!cm_FidCmp(fidp, &bp->fid) && (bp->flags & CM_BUF_DIRTY)) {
+		buf_Hold(bp);
+	    lock_ObtainMutex(&bp->mx);
+	    bp->cmFlags &= ~CM_BUF_CMSTORING;
+	    bp->flags &= ~CM_BUF_DIRTY;
+	    bp->flags |= CM_BUF_ERROR;
+	    bp->error = VNOVNODE;
+	    bp->dataVersion = -1; /* bad */
+	    bp->dirtyCounter++;
+	    if (bp->flags & CM_BUF_WAITING) {
+		osi_Log2(buf_logp, "BUF CleanDirtyBuffers Waking [scp 0x%x] bp 0x%x", scp, bp);
+		osi_Wakeup((long) &bp);
+	    }
+	    lock_ReleaseMutex(&bp->mx);
+	    buf_Release(bp);
+	}
+    }
+    return 0;
+}
+#endif
