@@ -16,14 +16,15 @@
 #include <string.h>
 #include <malloc.h>
 #include <osi.h>
+#include "afsd.h"
 #include <rx/rx.h>
 #include <rx/rxkad.h>
 #include <afs/unified_afs.h>
-#include "afsd.h"
+#include <WINNT/afsreg.h>
 
 osi_rwlock_t cm_connLock;
 
-long RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
+DWORD RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
 unsigned short ConnDeadtimeout = CM_CONN_CONNDEADTIME;
 unsigned short HardDeadtimeout = CM_CONN_HARDDEADTIME;
 
@@ -41,13 +42,14 @@ void cm_PutConn(cm_conn_t *connp)
 
 void cm_InitConn(void)
 {
-	static osi_once_t once;
-	long code;
-	DWORD sessTimeout;
-	HKEY parmKey;
+    static osi_once_t once;
+    long code;
+    DWORD dwValue;
+    DWORD dummyLen;
+    HKEY parmKey;
         
     if (osi_Once(&once)) {
-		lock_InitializeRWLock(&cm_connLock, "connection global lock");
+	lock_InitializeRWLock(&cm_connLock, "connection global lock");
 
         /* keisa - read timeout value for lanmanworkstation  service.
          * jaltman - as per 
@@ -56,29 +58,45 @@ void cm_InitConn(void)
          * I believe that the default should not be short.  Instead, we should wait until
          * RX times out before reporting a timeout to the SMB client.
          */
-		code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, LANMAN_WKS_PARAM_KEY,
-                            0, KEY_QUERY_VALUE, &parmKey);
-		if (code == ERROR_SUCCESS)
+	code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, LANMAN_WKS_PARAM_KEY,
+			    0, KEY_QUERY_VALUE, &parmKey);
+	if (code == ERROR_SUCCESS)
         {
-		    DWORD dummyLen = sizeof(sessTimeout);
-		    code = RegQueryValueEx(parmKey, LANMAN_WKS_SESSION_TIMEOUT, NULL, NULL, 
-                                   (BYTE *) &sessTimeout, &dummyLen);
-		    if (code == ERROR_SUCCESS)
-            {
-                afsi_log("lanmanworkstation : SessTimeout %d", sessTimeout);
-                RDRtimeout = sessTimeout;
-                if ( ConnDeadtimeout < RDRtimeout + 15 ) {
-                    ConnDeadtimeout = RDRtimeout + 15;
-                    afsi_log("ConnDeadTimeout increased to %d", ConnDeadtimeout);
-                }
-                if ( HardDeadtimeout < 2 * ConnDeadtimeout ) {
-                    HardDeadtimeout = 2 * ConnDeadtimeout;
-                    afsi_log("HardDeadTimeout increased to %d", HardDeadtimeout);
-                }
-            }
+	    dummyLen = sizeof(DWORD);
+	    code = RegQueryValueEx(parmKey, LANMAN_WKS_SESSION_TIMEOUT, NULL, NULL, 
+				   (BYTE *) &dwValue, &dummyLen);
+	    if (code == ERROR_SUCCESS)
+                RDRtimeout = dwValue;
+	    RegCloseKey(parmKey);
         }
 
-        osi_EndOnce(&once);
+	code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+			     0, KEY_QUERY_VALUE, &parmKey);
+	if (code == ERROR_SUCCESS) {
+	    dummyLen = sizeof(DWORD);
+	    code = RegQueryValueEx(parmKey, "ConnDeadTimeout", NULL, NULL,
+				    (BYTE *) &dwValue, &dummyLen);
+	    if (code == ERROR_SUCCESS)
+                ConnDeadtimeout = (unsigned short)dwValue;
+
+	    dummyLen = sizeof(DWORD);
+	    code = RegQueryValueEx(parmKey, "HardDeadTimeout", NULL, NULL,
+				    (BYTE *) &dwValue, &dummyLen);
+	    if (code == ERROR_SUCCESS)
+                HardDeadtimeout = (unsigned short)dwValue;
+	    afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
+	    RegCloseKey(parmKey);
+	}
+
+	afsi_log("lanmanworkstation : SessTimeout %u", RDRtimeout);
+	if (ConnDeadtimeout == 0)
+	    ConnDeadtimeout = (unsigned short) (RDRtimeout / 2);
+	afsi_log("ConnDeadTimeout is %d", ConnDeadtimeout);
+	if (HardDeadtimeout == 0)
+	    HardDeadtimeout = (unsigned short) RDRtimeout;
+	afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
+
+	osi_EndOnce(&once);
     }
 }
 
@@ -181,7 +199,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     if (reqp->flags & CM_REQ_NORETRY)
         goto out;
 
-    /* if timeout - check that it did not exceed the SMB timeout
+    /* if timeout - check that it did not exceed the HardDead timeout
      * and retry */
     
     /* timeleft - get if from reqp the same way as cmXonnByMServers does */
@@ -193,7 +211,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 #endif
 	    
     /* leave 5 seconds margin for sleep */
-    timeLeft = RDRtimeout - timeUsed;
+    timeLeft = HardDeadtimeout - timeUsed;
 
     if (errorCode == CM_ERROR_TIMEDOUT) {
         if (timeLeft > 5 ) {
@@ -420,12 +438,17 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 		if (scp->fileType != CM_SCACHETYPE_DIRECTORY)
 		    pscp = cm_FindSCacheParent(scp);
 
+
+		lock_ObtainMutex(&scp->mx);
 		lock_ObtainWrite(&cm_scacheLock);
-		cm_RecycleSCache(scp, CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS);
+		cm_RemoveSCacheFromHashTable(scp);
 		lock_ReleaseWrite(&cm_scacheLock);
+		scp->flags |= CM_SCACHEFLAG_DELETED;
+		lock_ReleaseMutex(&scp->mx);
+		cm_ReleaseSCache(scp);
 
  		if (pscp) {
-		    if (pscp->cbExpires > 0 && pscp->cbServerp != NULL) {
+		    if (cm_HaveCallback(pscp)) {
  			lock_ObtainMutex(&pscp->mx);
  			cm_DiscardSCache(pscp);
  			lock_ReleaseMutex(&pscp->mx);
@@ -470,8 +493,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
         if ( timeLeft > 2 )
             retry = 1;
     }
-    else if (errorCode == RXKADEXPIRED || 
-             errorCode == RXKADBADTICKET) {
+    else if (errorCode == RXKADEXPIRED || errorCode == RXKADBADTICKET) {
         if (!dead_session) {
             lock_ObtainMutex(&userp->mx);
             ucellp = cm_GetUCell(userp, serverp->cellp);
@@ -485,6 +507,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
             if ( timeLeft > 2 )
                 retry = 1;
         }
+    } else if (errorCode == VICECONNBAD || errorCode == VICETOKENDEAD) {
+	cm_ForceNewConnections(serverp);
+        if ( timeLeft > 2 )
+            retry = 1;
     } else {
         if (errorCode) {
             char * s = "unknown error";
@@ -522,9 +548,11 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
             case EINVAL            : s = "EINVAL";             break;
 	    case UAEINVAL          : s = "UAEINVAL";	       break;
             case EACCES            : s = "EACCES";             break;
-	    case UAEACCES 	   : s = "UAECCES";            break;
+	    case UAEACCES 	   : s = "UAEACCES";           break;
 	    case ENOENT            : s = "ENOENT"; 	       break;
 	    case UAENOENT          : s = "UAENOENT";           break;
+	    case VICECONNBAD	   : s = "VICECONNBAD";	       break;
+	    case VICETOKENDEAD     : s = "VICETOKENDEAD";      break;
 	    case CM_ERROR_NOSUCHCELL	    : s = "CM_ERROR_NOSUCHCELL";         break; 			
 	    case CM_ERROR_NOSUCHVOLUME	    : s = "CM_ERROR_NOSUCHVOLUME";       break; 			
 	    case CM_ERROR_TIMEDOUT	    : s = "CM_ERROR_TIMEDOUT";           break; 		
