@@ -21,11 +21,12 @@
 #include <malloc.h>
 #include <string.h>
 
+#include "afsd.h"
+
 #include <rx/rx.h>
 #include <rx/rx_prototypes.h>
 #include <WINNT/afsreg.h>
 
-#include "afsd.h"
 #include "afsicf.h"
 
 /* in seconds */
@@ -88,9 +89,14 @@ void cm_BkgDaemon(long parm)
         osi_assert(cm_bkgQueueCount-- > 0);
         lock_ReleaseWrite(&cm_daemonLock);
 
+#ifdef DEBUG_REFCOUNT
+	osi_Log2(afsd_logp,"cm_BkgDaemon (before) scp 0x%x ref %d",rp->scp, rp->scp->refCount);
+#endif
         (*rp->procp)(rp->scp, rp->p1, rp->p2, rp->p3, rp->p4, rp->userp);
-                
-        cm_ReleaseUser(rp->userp);
+#ifdef DEBUG_REFCOUNT                
+	osi_Log2(afsd_logp,"cm_BkgDaemon (after) scp 0x%x ref %d",rp->scp, rp->scp->refCount);
+#endif
+	cm_ReleaseUser(rp->userp);
         cm_ReleaseSCache(rp->scp);
         free(rp);
 
@@ -99,7 +105,7 @@ void cm_BkgDaemon(long parm)
     lock_ReleaseWrite(&cm_daemonLock);
 }
 
-void cm_QueueBKGRequest(cm_scache_t *scp, cm_bkgProc_t *procp, long p1, long p2, long p3, long p4,
+void cm_QueueBKGRequest(cm_scache_t *scp, cm_bkgProc_t *procp, afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4,
 	cm_user_t *userp)
 {
     cm_bkgRequest_t *rp;
@@ -143,7 +149,13 @@ IsWindowsFirewallPresent(void)
     if (!scm) return FALSE;
 
     /* Open Windows Firewall service */
-    svc = OpenService(scm, "SharedAccess", SERVICE_QUERY_CONFIG);
+    svc = OpenService(scm, "MpsSvc", SERVICE_QUERY_CONFIG);
+    if (!svc) {
+	afsi_log("MpsSvc Service could not be opened for query: 0x%x", GetLastError());
+	svc = OpenService(scm, "SharedAccess", SERVICE_QUERY_CONFIG);
+	if (!svc)
+	    afsi_log("SharedAccess Service could not be opened for query: 0x%x", GetLastError());
+    }
     if (!svc)
         goto close_scm;
 
@@ -161,10 +173,13 @@ IsWindowsFirewallPresent(void)
 
     /* Query Windows Firewall service config, this time for real */
     flag = QueryServiceConfig(svc, pConfig, BufSize, &BufSize);
-    if (!flag)
+    if (!flag) {
+	afsi_log("QueryServiceConfig failed: 0x%x", GetLastError());
         goto free_pConfig;
+    }
 
     /* Is it autostart? */
+    afsi_log("AutoStart 0x%x", pConfig->dwStartType);
     if (pConfig->dwStartType < SERVICE_DEMAND_START)
         result = TRUE;
 
@@ -246,6 +261,10 @@ void cm_Daemon(long parm)
     HMODULE hHookDll;
     int configureFirewall = IsWindowsFirewallPresent();
 
+    if (!configureFirewall) {
+	afsi_log("No Windows Firewall detected");
+    }
+
     /* ping all file servers, up or down, with unauthenticated connection,
      * to find out whether we have all our callbacks from the server still.
      * Also, ping down VLDBs.
@@ -277,9 +296,10 @@ void cm_Daemon(long parm)
     lastTokenCacheCheck = now - cm_daemonTokenCheckInterval/2 + (rand() % cm_daemonTokenCheckInterval);
 
     while (daemon_ShutdownFlag == 0) {
-	thrd_Sleep(30 * 1000);		/* sleep 30 seconds */
-        if (daemon_ShutdownFlag == 1)
-            return;
+	/* check to see if the listener threads halted due to network 
+	 * disconnect or other issues.  If so, attempt to restart them.
+	 */
+	smb_RestartListeners();
 
 	if (configureFirewall) {
 	    /* Open Microsoft Firewall to allow in port 7001 */
@@ -300,7 +320,7 @@ void cm_Daemon(long parm)
 	    default:
 		afsi_log("Unknown Windows Firewall Configuration error");
 	    }
-	}
+	} 
 
         /* find out what time it is */
         now = osi_Time();
@@ -363,6 +383,10 @@ void cm_Daemon(long parm)
                 SetEvent(WaitToTerminate);
             }
         }
+
+	thrd_Sleep(30 * 1000);		/* sleep 30 seconds */
+        if (daemon_ShutdownFlag == 1)
+            return;
     }
 }       
 
