@@ -33,10 +33,20 @@ osi_mutex_t cm_bufGetMutex;
 extern osi_mutex_t cm_Freelance_Lock;
 #endif
 
+#ifdef AFS_LARGEFILES
+/* we can access connp->serverp without holding a lock because that
+   never changes since the connection is made. */
+#define SERVERHAS64BIT(connp) (!((connp)->serverp->flags & CM_SERVERFLAG_NO64BIT))
+#define SET_SERVERHASNO64BIT(connp) (cm_SetServerNo64Bit((connp)->serverp, TRUE))
+#else
+#define SERVERHAS64BIT(connp) (FALSE)
+#define SET_SERVERHASNO64BIT(connp) (FALSE)
+#endif
+
 /* functions called back from the buffer package when reading or writing data,
  * or when holding or releasing a vnode pointer.
  */
-long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
+long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
                  cm_user_t *userp, cm_req_t *reqp)
 {
     /* store the data back from this buffer; the buffer is locked and held,
@@ -46,8 +56,7 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
      * bufp->scp.
      */
     long code;
-    cm_fid_t *fidp = vfidp;
-    cm_scache_t *scp;
+    cm_scache_t *scp = vscp;
     long nbytes;
     long temp;
     AFSFetchStatus outStatus;
@@ -62,28 +71,30 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
     long wbytes;
     char *bufferp;
     cm_conn_t *connp;
-    long truncPos;
+    osi_hyper_t truncPos;
     cm_bulkIO_t biod;		/* bulk IO descriptor */
+    int require_64bit_ops = 0;
 
     osi_assert(userp != NULL);
+    osi_assert(scp != NULL);
 
     /* now, the buffer may or may not be filled with good data (buf_GetNew
      * drops lots of locks, and may indeed return a properly initialized
      * buffer, although more likely it will just return a new, empty, buffer.
      */
-    scp = cm_FindSCache(fidp);
-    if (scp == NULL)
-        return CM_ERROR_NOSUCHFILE;	/* shouldn't happen */
-
-    cm_AFSFidFromFid(&tfid, fidp);
 
     lock_ObtainMutex(&scp->mx);
-        
+    cm_AFSFidFromFid(&tfid, &scp->fid);
+
+    if (scp->flags & CM_SCACHEFLAG_DELETED) {
+	lock_ReleaseMutex(&scp->mx);
+	return CM_ERROR_NOSUCHFILE;
+    }
+
     code = cm_SetupStoreBIOD(scp, offsetp, length, &biod, userp, reqp);
     if (code) {
         osi_Log1(afsd_logp, "cm_SetupStoreBIOD code %x", code);
         lock_ReleaseMutex(&scp->mx);
-        cm_ReleaseSCache(scp);
         return code;
     }
 
@@ -91,9 +102,8 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
         osi_Log0(afsd_logp, "cm_SetupStoreBIOD length 0");
         lock_ReleaseMutex(&scp->mx);
         cm_ReleaseBIOD(&biod, 1);	/* should be a NOOP */
-        cm_ReleaseSCache(scp);
         return 0;
-    }   
+    }
 
     /* Serialize StoreData RPC's; for rationale see cm_scache.c */
     (void) cm_SyncOp(scp, NULL, userp, reqp, 0, CM_SCACHESYNC_STOREDATA_EXCL);
@@ -101,10 +111,10 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
     /* prepare the output status for the store */
     scp->mask |= CM_SCACHEMASK_CLIENTMODTIME;
     cm_StatusFromAttr(&inStatus, scp, NULL);
-    truncPos = scp->length.LowPart;
+    truncPos = scp->length;
     if ((scp->mask & CM_SCACHEMASK_TRUNCPOS)
-         && scp->truncPos.LowPart < (unsigned long) truncPos)
-        truncPos = scp->truncPos.LowPart;
+        && LargeIntegerLessThan(scp->truncPos, truncPos))
+        truncPos = scp->truncPos;
 	scp->mask &= ~CM_SCACHEMASK_TRUNCPOS;
                 
     /* compute how many bytes to write from this buffer */
@@ -117,28 +127,66 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
         /* otherwise write out part of buffer before EOF, but not
          * more than bufferSize bytes.
          */
-        nbytes = thyper.LowPart;
-        if (nbytes > biod.length) 
+        if (LargeIntegerGreaterThan(thyper,
+                                    ConvertLongToLargeInteger(biod.length))) {
             nbytes = biod.length;
+        } else {
+            /* if thyper is less than or equal to biod.length, then we
+               can safely assume that the value fits in a long. */
+            nbytes = thyper.LowPart;
+        }
     }
 
-    lock_ReleaseMutex(&scp->mx);
+    if (LargeIntegerGreaterThan(LargeIntegerAdd(biod.offset,
+                                                 ConvertLongToLargeInteger(nbytes)),
+                                 ConvertLongToLargeInteger(LONG_MAX)) ||
+         LargeIntegerGreaterThan(truncPos,
+                                 ConvertLongToLargeInteger(LONG_MAX))) {
+        require_64bit_ops = 1;
+    }
         
+    lock_ReleaseMutex(&scp->mx);
+
     /* now we're ready to do the store operation */
     do {
         code = cm_Conn(&scp->fid, userp, reqp, &connp);
         if (code) 
             continue;
-		
+
+    retry:
         rxconnp = cm_GetRxConn(connp);
         callp = rx_NewCall(rxconnp);
         rx_PutConnection(rxconnp);
 
+#ifdef AFS_LARGEFILES
+        if (SERVERHAS64BIT(connp)) {
+            osi_Log4(afsd_logp, "CALL StoreData64 scp 0x%p, offset 0x%x:%08x, length 0x%x",
+                     scp, biod.offset.HighPart, biod.offset.LowPart, nbytes);
+
+            code = StartRXAFS_StoreData64(callp, &tfid, &inStatus,
+                                          biod.offset.QuadPart,
+                                          nbytes,
+                                          truncPos.QuadPart);
+        } else {
+
+            if (require_64bit_ops) {
+                osi_Log0(afsd_logp, "Skipping StoreData.  The operation requires StoreData64");
+                code = CM_ERROR_TOOBIG;
+            } else {
+                osi_Log4(afsd_logp, "CALL StoreData scp 0x%p, offset 0x%x:%08x, length 0x%x",
+                         scp, biod.offset.HighPart, biod.offset.LowPart, nbytes);
+
+                code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
+                                            biod.offset.LowPart, nbytes, truncPos.LowPart);
+            }
+        }
+#else
         osi_Log4(afsd_logp, "CALL StoreData scp 0x%p, offset 0x%x:%08x, length 0x%x",
                  scp, biod.offset.HighPart, biod.offset.LowPart, nbytes);
 
         code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
-                                    biod.offset.LowPart, nbytes, truncPos);
+                                    biod.offset.LowPart, nbytes, truncPos.LowPart);
+#endif
 
         if (code == 0) {
             /* write the data from the the list of buffers */
@@ -168,18 +216,34 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
             }	/* while more bytes to write */
         }		/* if RPC started successfully */
         else {
-            osi_Log2(afsd_logp, "StartRXAFS_StoreData scp 0x%p failed (%lX)",scp,code);
+            osi_Log2(afsd_logp, "StartRXAFS_StoreData?? scp 0x%p failed (%lX)",scp,code);
         }
+
         if (code == 0) {
-            code = EndRXAFS_StoreData(callp, &outStatus, &volSync);
-            if (code)
-                osi_Log2(afsd_logp, "EndRXAFS_StoreData scp 0x%p failed (%lX)",scp,code);
+            if (SERVERHAS64BIT(connp)) {
+                code = EndRXAFS_StoreData64(callp, &outStatus, &volSync);
+                if (code)
+                    osi_Log2(afsd_logp, "EndRXAFS_StoreData64 scp 0x%p failed (%lX)", scp, code);
+            } else {
+                code = EndRXAFS_StoreData(callp, &outStatus, &volSync);
+                if (code)
+                    osi_Log2(afsd_logp, "EndRXAFS_StoreData scp 0x%p failed (%lX)",scp,code);
+            }
         }
+
         code = rx_EndCall(callp, code);
+
+#ifdef AFS_LARGEFILES
+        if (code == RXGEN_OPCODE && SERVERHAS64BIT(connp)) {
+            SET_SERVERHASNO64BIT(connp);
+            goto retry;
+        }
+#endif
                 
     } while (cm_Analyze(connp, userp, reqp, &scp->fid, &volSync, NULL, NULL, code));
+
     code = cm_MapRPCError(code, reqp);
-        
+
     if (code)
         osi_Log2(afsd_logp, "CALL StoreData FAILURE scp 0x%p, code 0x%x", scp, code);
     else
@@ -191,6 +255,7 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_STOREDATA_EXCL);
 
     if (code == 0) {
+        osi_hyper_t t;
         /* now, here's something a little tricky: in AFS 3, a dirty
          * length can't be directly stored, instead, a dirty chunk is
          * stored that sets the file's size (by writing and by using
@@ -209,8 +274,17 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
          * We have to turn off the length dirty bit as soon as we can,
          * so that we see updates made by other machines.
          */
-        if (outStatus.Length >= scp->length.LowPart)
+
+        if (SERVERHAS64BIT(connp)) {
+            t.LowPart = outStatus.Length;
+            t.HighPart = outStatus.Length_hi;
+        } else {
+            t = ConvertLongToLargeInteger(outStatus.Length);
+        }
+
+        if (LargeIntegerGreaterThanOrEqualTo(t, scp->length))
             scp->mask &= ~CM_SCACHEMASK_LENGTH;
+
         cm_MergeStatus(scp, &outStatus, &volSync, userp, 0);
     } else {
         if (code == CM_ERROR_SPACE)
@@ -220,7 +294,6 @@ long cm_BufWrite(void *vfidp, osi_hyper_t *offsetp, long length, long flags,
     }
     lock_ReleaseMutex(&scp->mx);
     cm_ReleaseBIOD(&biod, 1);
-    cm_ReleaseSCache(scp);
 
     return code;
 }
@@ -237,10 +310,11 @@ long cm_StoreMini(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     AFSVolSync volSync;
     AFSFid tfid;
     long code;
-    long truncPos;
+    osi_hyper_t truncPos;
     cm_conn_t *connp;
     struct rx_call *callp;
     struct rx_connection *rxconnp;
+    int require_64bit_ops = 0;
 
     /* Serialize StoreData RPC's; for rationale see cm_scache.c */
     (void) cm_SyncOp(scp, NULL, userp, reqp, 0,
@@ -252,12 +326,18 @@ long cm_StoreMini(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     scp->mask &= ~CM_SCACHEMASK_CLIENTMODTIME;
 
     /* calculate truncation position */
-    truncPos = scp->length.LowPart;
+    truncPos = scp->length;
     if ((scp->mask & CM_SCACHEMASK_TRUNCPOS)
-         && scp->truncPos.LowPart < (unsigned long) truncPos)
-        truncPos = scp->truncPos.LowPart;
+        && LargeIntegerLessThan(scp->truncPos, truncPos))
+        truncPos = scp->truncPos;
     scp->mask &= ~CM_SCACHEMASK_TRUNCPOS;
 
+    if (LargeIntegerGreaterThan(truncPos,
+                                ConvertLongToLargeInteger(LONG_MAX))) {
+
+        require_64bit_ops = 1;
+    }
+
     lock_ReleaseMutex(&scp->mx);
 
     cm_AFSFidFromFid(&tfid, &scp->fid);
@@ -267,18 +347,44 @@ long cm_StoreMini(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
         code = cm_Conn(&scp->fid, userp, reqp, &connp);
         if (code) 
             continue;
-		
+
+    retry:	
         rxconnp = cm_GetRxConn(connp);
         callp = rx_NewCall(rxconnp);
         rx_PutConnection(rxconnp);
 
+#ifdef AFS_LARGEFILES
+        if (SERVERHAS64BIT(connp)) {
+            code = StartRXAFS_StoreData64(callp, &tfid, &inStatus,
+                                          0, 0, truncPos.QuadPart);
+        } else {
+            if (require_64bit_ops) {
+                code = CM_ERROR_TOOBIG;
+            } else {
+                code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
+                                            0, 0, truncPos.LowPart);
+            }
+        }
+#else
         code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
-                                    0, 0, truncPos);
+                                    0, 0, truncPos.LowPart);
+#endif
 
-        if (code == 0)
-            code = EndRXAFS_StoreData(callp, &outStatus, &volSync);
+        if (code == 0) {
+            if (SERVERHAS64BIT(connp))
+                code = EndRXAFS_StoreData64(callp, &outStatus, &volSync);
+            else
+                code = EndRXAFS_StoreData(callp, &outStatus, &volSync);
+        }
         code = rx_EndCall(callp, code);
 
+#ifdef AFS_LARGEFILES
+        if (code == RXGEN_OPCODE && SERVERHAS64BIT(connp)) {
+            SET_SERVERHASNO64BIT(connp);
+            goto retry;
+        }
+#endif
+
     } while (cm_Analyze(connp, userp, reqp, &scp->fid, &volSync, NULL, NULL, code));
     code = cm_MapRPCError(code, reqp);
         
@@ -288,11 +394,19 @@ long cm_StoreMini(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_STOREDATA_EXCL);
 
     if (code == 0) {
+        osi_hyper_t t;
         /*
          * For explanation of handling of CM_SCACHEMASK_LENGTH,
          * see cm_BufWrite().
          */
-        if (outStatus.Length >= scp->length.LowPart)
+        if (SERVERHAS64BIT(connp)) {
+            t.HighPart = outStatus.Length_hi;
+            t.LowPart = outStatus.Length;
+        } else {
+            t = ConvertLongToLargeInteger(outStatus.Length);
+        }
+
+        if (LargeIntegerGreaterThanOrEqualTo(t, scp->length))
             scp->mask &= ~CM_SCACHEMASK_LENGTH;
         cm_MergeStatus(scp, &outStatus, &volSync, userp, 0);
     }
@@ -311,13 +425,11 @@ long cm_BufRead(cm_buf_t *bufp, long nbytes, long *bytesReadp, cm_user_t *userp)
 /* stabilize scache entry, and return with it locked so 
  * it stays stable.
  */
-long cm_BufStabilize(void *parmp, cm_user_t *userp, cm_req_t *reqp)
+long cm_BufStabilize(void *vscp, cm_user_t *userp, cm_req_t *reqp)
 {
-    cm_scache_t *scp;
+    cm_scache_t *scp = vscp;
     long code;
 
-    scp = parmp;
-        
     lock_ObtainMutex(&scp->mx);
     code = cm_SyncOp(scp, NULL, userp, reqp, 0, 
                      CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS | CM_SCACHESYNC_SETSIZE);
@@ -330,12 +442,12 @@ long cm_BufStabilize(void *parmp, cm_user_t *userp, cm_req_t *reqp)
 }
 
 /* undoes the work that cm_BufStabilize does: releases lock so things can change again */
-long cm_BufUnstabilize(void *parmp, cm_user_t *userp)
+long cm_BufUnstabilize(void *vscp, cm_user_t *userp)
 {
-    cm_scache_t *scp;
-        
-    scp = parmp;
+    cm_scache_t *scp = vscp;
         
+    cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS | CM_SCACHESYNC_SETSIZE);
+
     lock_ReleaseMutex(&scp->mx);
         
     /* always succeeds */
@@ -445,6 +557,7 @@ long cm_CheckFetchRange(cm_scache_t *scp, osi_hyper_t *startBasep, long length,
                  && bp->dataVersion != scp->dataVersion)
                 stop = 1;
             buf_Release(bp);
+	    bp = NULL;
         }
         else 
             stop = 1;
@@ -478,23 +591,30 @@ long cm_CheckFetchRange(cm_scache_t *scp, osi_hyper_t *startBasep, long length,
     return code;
 }
 
-void cm_BkgStore(cm_scache_t *scp, long p1, long p2, long p3, long p4,
+void cm_BkgStore(cm_scache_t *scp, afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4,
                  cm_user_t *userp)
 {
     osi_hyper_t toffset;
     long length;
     cm_req_t req;
+    long code;
 
-    cm_InitReq(&req);
-    req.flags |= CM_REQ_NORETRY;
+    if (scp->flags & CM_SCACHEFLAG_DELETED) {
+	osi_Log4(afsd_logp, "Skipping BKG store - Deleted scp 0x%p, offset 0x%x:%08x, length 0x%x", scp, p2, p1, p3);
+    } else {
+	cm_InitReq(&req);
+#ifdef NO_BKG_RETRIES
+	req.flags |= CM_REQ_NORETRY;
+#endif
 
-    toffset.LowPart = p1;
-    toffset.HighPart = p2;
-    length = p3;
+	toffset.LowPart = p1;
+	toffset.HighPart = p2;
+	length = p3;
 
-    osi_Log4(afsd_logp, "Starting BKG store scp 0x%p, offset 0x%x:%08x, length 0x%x", scp, p2, p1, p3);
+	osi_Log4(afsd_logp, "Starting BKG store scp 0x%p, offset 0x%x:%08x, length 0x%x", scp, p2, p1, p3);
 
-    cm_BufWrite(&scp->fid, &toffset, length, /* flags */ 0, userp, &req);
+	code = cm_BufWrite(scp, &toffset, length, /* flags */ 0, userp, &req);
+    }
 
     lock_ObtainMutex(&scp->mx);
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_ASYNCSTORE);
@@ -520,7 +640,7 @@ void cm_ClearPrefetchFlag(long code, cm_scache_t *scp, osi_hyper_t *base)
 }
 
 /* do the prefetch */
-void cm_BkgPrefetch(cm_scache_t *scp, long p1, long p2, long p3, long p4,
+void cm_BkgPrefetch(cm_scache_t *scp, afs_uint32 p1, afs_uint32 p2, afs_uint32 p3, afs_uint32 p4,
                     cm_user_t *userp)
 {
     long length;
@@ -626,7 +746,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
     long flags;			/* flags to cm_SyncOp */
         
     /* clear things out */
-    biop->scp = scp;		/* don't hold */
+    biop->scp = scp;			/* do not hold; held by caller */
     biop->offset = *inOffsetp;
     biop->length = 0;
     biop->bufListp = NULL;
@@ -639,7 +759,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
     lock_ObtainMutex(&scp->mx);
 
     bufp = NULL;
-    for (temp = 0; temp < inSize; temp += cm_data.buf_blockSize, bufp = NULL) {
+    for (temp = 0; temp < inSize; temp += cm_data.buf_blockSize) {
         thyper.HighPart = 0;
         thyper.LowPart = temp;
         tbase = LargeIntegerAdd(*inOffsetp, thyper);
@@ -651,10 +771,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
             lock_ObtainMutex(&bufp->mx);
             lock_ObtainMutex(&scp->mx);
 
-            flags = CM_SCACHESYNC_NEEDCALLBACK
-                | CM_SCACHESYNC_GETSTATUS
-                    | CM_SCACHESYNC_STOREDATA
-                        | CM_SCACHESYNC_BUFLOCKED;
+            flags = CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS | CM_SCACHESYNC_STOREDATA | CM_SCACHESYNC_BUFLOCKED;
             code = cm_SyncOp(scp, bufp, userp, reqp, 0, flags); 
             if (code) {
                 lock_ReleaseMutex(&bufp->mx);
@@ -675,6 +792,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
             cm_SyncOpDone(scp, bufp, flags);
             lock_ReleaseMutex(&bufp->mx);
             buf_Release(bufp);
+	    bufp = NULL;
         }       
     }
 
@@ -700,6 +818,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
     biop->length = cm_data.buf_blockSize;
     firstModOffset = bufp->offset;
     biop->offset = firstModOffset;
+    bufp = NULL;	/* this buffer and reference added to the queue */
 
     /* compute the window surrounding *inOffsetp of size cm_chunkSize */
     scanStart = *inOffsetp;
@@ -708,8 +827,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
     thyper.HighPart = 0;
     scanEnd = LargeIntegerAdd(scanStart, thyper);
 
-    flags = CM_SCACHESYNC_NEEDCALLBACK
-        | CM_SCACHESYNC_GETSTATUS
+    flags = CM_SCACHESYNC_GETSTATUS
         | CM_SCACHESYNC_STOREDATA
         | CM_SCACHESYNC_BUFLOCKED
         | CM_SCACHESYNC_NOWAIT;
@@ -730,6 +848,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
         lock_ObtainMutex(&scp->mx);
         if (code == 0) {
             buf_Release(bufp);
+	    bufp = NULL;
             break;
         }
                 
@@ -737,6 +856,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
         if (code) {
             lock_ReleaseMutex(&bufp->mx);
             buf_Release(bufp);
+	    bufp = NULL;
             break;
         }
                 
@@ -745,6 +865,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
             cm_SyncOpDone(scp, bufp, flags);
             lock_ReleaseMutex(&bufp->mx);
             buf_Release(bufp);
+	    bufp = NULL;
             break;
         }
 
@@ -761,6 +882,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
         osi_QAddT((osi_queue_t **) &biop->bufListp,
                   (osi_queue_t **) &biop->bufListEndp,
                   &qdp->q);
+	bufp = NULL;		/* added to the queue */
 
         /* update biod info describing the transfer */
         biop->offset = LargeIntegerSubtract(biop->offset, thyper);
@@ -786,6 +908,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
         lock_ObtainMutex(&scp->mx);
         if (code == 0) {
             buf_Release(bufp);
+	    bufp = NULL;
             break;
         }
 
@@ -793,6 +916,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
         if (code) {
             lock_ReleaseMutex(&bufp->mx);
             buf_Release(bufp);
+	    bufp = NULL;
             break;
         }
                 
@@ -801,6 +925,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
             cm_SyncOpDone(scp, bufp, flags);
             lock_ReleaseMutex(&bufp->mx);
             buf_Release(bufp);
+	    bufp = NULL;
             break;
         }
 
@@ -817,6 +942,7 @@ long cm_SetupStoreBIOD(cm_scache_t *scp, osi_hyper_t *inOffsetp, long inSize,
         osi_QAddH((osi_queue_t **) &biop->bufListp,
                   (osi_queue_t **) &biop->bufListEndp,
                   &qdp->q);
+	bufp = NULL;
 
         /* update biod info describing the transfer */
         biop->length += cm_data.buf_blockSize;
@@ -851,7 +977,7 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
     osi_queueData_t *heldBufListEndp;	/* first one */
     int reserving;
 
-    biop->scp = scp;
+    biop->scp = scp;			/* do not hold; held by caller */
     biop->offset = *offsetp;
     /* null out the list of buffers */
     biop->bufListp = biop->bufListEndp = NULL;
@@ -944,8 +1070,9 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
         /* add the buffer to the list */
         qdp = osi_QDAlloc();
         osi_SetQData(qdp, tbp);
-        osi_QAdd((osi_queue_t **)&heldBufListp, &qdp->q);
-        if (!heldBufListEndp) heldBufListEndp = qdp;
+        osi_QAddH((osi_queue_t **)&heldBufListp, 
+		  (osi_queue_t **)&heldBufListEndp, 
+		  &qdp->q);
         /* leave tbp held (from buf_Get) */
 
         if (!reserving) 
@@ -983,8 +1110,7 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
             break;
         }
 
-        flags = CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_FETCHDATA
-            | CM_SCACHESYNC_BUFLOCKED;
+        flags = CM_SCACHESYNC_FETCHDATA | CM_SCACHESYNC_BUFLOCKED;
         if (!isFirst) 
             flags |= CM_SCACHESYNC_NOWAIT;
 
@@ -1013,9 +1139,9 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
         /* add the buffer to the list */
         qdp = osi_QDAlloc();
         osi_SetQData(qdp, tbp);
-        osi_QAdd((osi_queue_t **)&biop->bufListp, &qdp->q);
-        if (!biop->bufListEndp) 
-            biop->bufListEndp = qdp;
+        osi_QAddH((osi_queue_t **)&biop->bufListp, 
+		  (osi_queue_t **)&biop->bufListEndp, 
+		  &qdp->q);
         buf_Hold(tbp);
 
         /* from now on, a failure just stops our collection process, but
@@ -1032,6 +1158,9 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
     for (qdp = heldBufListp; qdp; qdp = tqdp) {
         tqdp = (osi_queueData_t *) osi_QNext(&qdp->q);
         tbp = osi_GetQData(qdp);
+	osi_QRemoveHT((osi_queue_t **) &heldBufListp,
+		      (osi_queue_t **) &heldBufListEndp,
+		      &qdp->q);
         osi_QDFree(qdp);
         buf_Release(tbp);
     }
@@ -1061,7 +1190,7 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
  */
 void cm_ReleaseBIOD(cm_bulkIO_t *biop, int isStore)
 {
-    cm_scache_t *scp;
+    cm_scache_t *scp;		/* do not release; not held in biop */
     cm_buf_t *bufp;
     osi_queueData_t *qdp;
     osi_queueData_t *nqdp;
@@ -1071,38 +1200,47 @@ void cm_ReleaseBIOD(cm_bulkIO_t *biop, int isStore)
     if (biop->reserved)
         buf_UnreserveBuffers(cm_chunkSize / cm_data.buf_blockSize);
         
-    flags = CM_SCACHESYNC_NEEDCALLBACK;
     if (isStore)
-        flags |= CM_SCACHESYNC_STOREDATA;
+        flags = CM_SCACHESYNC_STOREDATA;
     else
-        flags |= CM_SCACHESYNC_FETCHDATA;
+        flags = CM_SCACHESYNC_FETCHDATA;
 
     scp = biop->scp;
-    for(qdp = biop->bufListp; qdp; qdp = nqdp) {
-        /* lookup next guy first, since we're going to free this one */
-        nqdp = (osi_queueData_t *) osi_QNext(&qdp->q);
+    if (biop->bufListp) {
+	for(qdp = biop->bufListp; qdp; qdp = nqdp) {
+	    /* lookup next guy first, since we're going to free this one */
+	    nqdp = (osi_queueData_t *) osi_QNext(&qdp->q);
                 
-        /* extract buffer and free queue data */
-        bufp = osi_GetQData(qdp);
-        osi_QDFree(qdp);
+	    /* extract buffer and free queue data */
+	    bufp = osi_GetQData(qdp);
+	    osi_QRemoveHT((osi_queue_t **) &biop->bufListp,
+			   (osi_queue_t **) &biop->bufListEndp,
+			   &qdp->q);
+	    osi_QDFree(qdp);
 
-        /* now, mark I/O as done, unlock the buffer and release it */
-        lock_ObtainMutex(&bufp->mx);
-        lock_ObtainMutex(&scp->mx);
-        cm_SyncOpDone(scp, bufp, flags);
+	    /* now, mark I/O as done, unlock the buffer and release it */
+	    lock_ObtainMutex(&bufp->mx);
+	    lock_ObtainMutex(&scp->mx);
+	    cm_SyncOpDone(scp, bufp, flags);
                 
-        /* turn off writing and wakeup users */
-        if (isStore) {
-            if (bufp->flags & CM_BUF_WAITING) {
-                osi_Log2(afsd_logp, "cm_ReleaseBIOD Waking [scp 0x%p] bp 0x%p", scp, bufp);
-                osi_Wakeup((LONG_PTR) bufp);
-            }
-            bufp->flags &= ~(CM_BUF_WRITING | CM_BUF_DIRTY);
-        }
+	    /* turn off writing and wakeup users */
+	    if (isStore) {
+		if (bufp->flags & CM_BUF_WAITING) {
+		    osi_Log2(afsd_logp, "cm_ReleaseBIOD Waking [scp 0x%p] bp 0x%p", scp, bufp);
+		    osi_Wakeup((LONG_PTR) bufp);
+		}
+		bufp->flags &= ~(CM_BUF_WRITING | CM_BUF_DIRTY);
+	    }
 
-        lock_ReleaseMutex(&scp->mx);
-        lock_ReleaseMutex(&bufp->mx);
-        buf_Release(bufp);
+	    lock_ReleaseMutex(&scp->mx);
+	    lock_ReleaseMutex(&bufp->mx);
+	    buf_Release(bufp);
+	    bufp = NULL;
+	}
+    } else {
+	lock_ObtainMutex(&scp->mx);
+	cm_SyncOpDone(scp, NULL, flags);
+	lock_ReleaseMutex(&scp->mx);
     }
 
     /* clean things out */
@@ -1117,7 +1255,9 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *up,
                   cm_req_t *reqp)
 {
     long code;
-    long nbytes;			/* bytes in transfer */
+    afs_int32 nbytes;			/* bytes in transfer */
+    afs_int32 nbytes_hi = 0;            /* high-order 32 bits of bytes in transfer */
+    afs_int64 length_found = 0;
     long rbytes;			/* bytes in rx_Read call */
     long temp;
     AFSFetchStatus afsStatus;
@@ -1133,6 +1273,7 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *up,
     cm_conn_t *connp;
     int getroot;
     long t1, t2;
+    int require_64bit_ops = 0;
 
     /* now, the buffer may or may not be filled with good data (buf_GetNew
      * drops lots of locks, and may indeed return a properly initialized
@@ -1182,9 +1323,15 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *up,
         lock_ObtainMutex(&scp->mx);
         return 0;
     }
-        
+
     lock_ReleaseMutex(&scp->mx);
 
+    if (LargeIntegerGreaterThan(LargeIntegerAdd(biod.offset,
+                                                ConvertLongToLargeInteger(biod.length)),
+                                ConvertLongToLargeInteger(LONG_MAX))) {
+        require_64bit_ops = 1;
+    }
+
 #ifdef DISKCACHE95
     DPRINTF("cm_GetBuffer: fetching data scpDV=%d bufDV=%d scp=%x bp=%x dcp=%x\n",
             scp->dataVersion, bufp->dataVersion, scp, bufp, bufp->dcp);
@@ -1259,11 +1406,68 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *up,
         code = cm_Conn(&scp->fid, up, reqp, &connp);
         if (code) 
             continue;
-	
+
         rxconnp = cm_GetRxConn(connp);
         callp = rx_NewCall(rxconnp);
         rx_PutConnection(rxconnp);
 
+#ifdef AFS_LARGEFILES
+        nbytes = nbytes_hi = 0;
+
+        if (SERVERHAS64BIT(connp)) {
+            osi_Log4(afsd_logp, "CALL FetchData64 scp 0x%p, off 0x%x:%08x, size 0x%x",
+                     scp, biod.offset.HighPart, biod.offset.LowPart, biod.length);
+
+            code = StartRXAFS_FetchData64(callp, &tfid, biod.offset.QuadPart, biod.length);
+
+            if (code == 0) {
+                temp = rx_Read(callp, (char *) &nbytes_hi, sizeof(afs_int32));
+                if (temp == sizeof(afs_int32)) {
+                    nbytes_hi = ntohl(nbytes_hi);
+                } else {
+                    nbytes_hi = 0;
+		    code = callp->error;
+                    rx_EndCall(callp, code);
+                    callp = NULL;
+                }
+            }
+        }
+
+        if (code == RXGEN_OPCODE || !SERVERHAS64BIT(connp)) {
+            if (require_64bit_ops) {
+                osi_Log0(afsd_logp, "Skipping FetchData.  Operation requires FetchData64");
+                code = CM_ERROR_TOOBIG;
+            } else {
+                if (!callp) {
+                    rxconnp = cm_GetRxConn(connp);
+                    callp = rx_NewCall(rxconnp);
+                    rx_PutConnection(rxconnp);
+                }
+
+                osi_Log3(afsd_logp, "CALL FetchData scp 0x%p, off 0x%x, size 0x%x",
+                         scp, biod.offset.LowPart, biod.length);
+
+                code = StartRXAFS_FetchData(callp, &tfid, biod.offset.LowPart,
+                                            biod.length);
+
+                SET_SERVERHASNO64BIT(connp);
+            }
+        }
+
+        if (code == 0) {
+            temp  = rx_Read(callp, (char *)&nbytes, sizeof(afs_int32));
+            if (temp == sizeof(afs_int32)) {
+                nbytes = ntohl(nbytes);
+                FillInt64(length_found, nbytes_hi, nbytes);
+                if (length_found > biod.length) 
+                    code = (callp->error < 0) ? callp->error : -1;
+            } else {
+                code = (callp->error < 0) ? callp->error : -1;
+            }
+        }
+        /* for the moment, nbytes_hi will always be 0 if code == 0
+           because biod.length is a 32-bit quantity. */
+#else
         osi_Log3(afsd_logp, "CALL FetchData scp 0x%p, off 0x%x, size 0x%x",
                  scp, biod.offset.LowPart, biod.length);
 
@@ -1271,14 +1475,17 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *up,
                                     biod.length);
 
         /* now copy the data out of the pipe and put it in the buffer */
-        temp  = rx_Read(callp, (char *)&nbytes, 4);
-        if (temp == 4) {
-            nbytes = ntohl(nbytes);
-            if (nbytes > biod.length) 
+        if (code == 0) {
+            temp  = rx_Read(callp, (char *)&nbytes, sizeof(afs_int32));
+            if (temp == sizeof(afs_int32)) {
+                nbytes = ntohl(nbytes);
+                if (nbytes > biod.length) 
+                    code = (callp->error < 0) ? callp->error : -1;
+            }
+            else 
                 code = (callp->error < 0) ? callp->error : -1;
         }
-        else 
-            code = (callp->error < 0) ? callp->error : -1;
+#endif
 
         if (code == 0) {
             qdp = biod.bufListEndp;
@@ -1368,13 +1575,24 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *up,
             }   
         }
 
-        if (code == 0)
-            code = EndRXAFS_FetchData(callp, &afsStatus, &callback, &volSync);
-        else
-            osi_Log1(afsd_logp, "CALL EndRXAFS_FetchData skipped due to error %d", code);
-        code = rx_EndCall(callp, code);
+        if (code == 0) {
+            if (SERVERHAS64BIT(connp))
+                code = EndRXAFS_FetchData64(callp, &afsStatus, &callback, &volSync);
+            else
+                code = EndRXAFS_FetchData(callp, &afsStatus, &callback, &volSync);
+        } else {
+            if (SERVERHAS64BIT(connp))
+                osi_Log1(afsd_logp, "CALL EndRXAFS_FetchData64 skipped due to error %d", code);
+            else
+                osi_Log1(afsd_logp, "CALL EndRXAFS_FetchData skipped due to error %d", code);
+        }
+
+        if (callp)
+            code = rx_EndCall(callp, code);
+
         if (code == RXKADUNKNOWNKEY)
             osi_Log0(afsd_logp, "CALL EndCall returns RXKADUNKNOWNKEY");
+
         osi_Log0(afsd_logp, "CALL FetchData DONE");
 
     } while (cm_Analyze(connp, up, reqp, &scp->fid, &volSync, NULL, NULL, code));
