@@ -216,14 +216,11 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 	cm_scache_t **scpp)
 {
     long code;
-    cm_scache_t *substRootp;
-    char * relativePath = ioctlp->inDatap;
-#ifdef AFSIFS
-    char * absRoot[MAX_PATH];
-    long length;
-    wchar_t absRoot_w[MAX_PATH];
-    HANDLE rootDir;
+#ifndef AFSIFS
+    cm_scache_t *substRootp = NULL;
 #endif
+    char * relativePath = ioctlp->inDatap;
+
     osi_Log1(afsd_logp, "cm_ParseIoctlPath %s", osi_LogSaveString(afsd_logp,relativePath));
 
     /* This is usually the file name, but for StatMountPoint it is the path. */
@@ -237,26 +234,18 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 
 #ifdef AFSIFS
     /* we have passed the whole path, including the afs prefix.
-	   when the pioctl call is made, we perform an ioctl to afsrdr
-	   and it returns the correct (full) path.  therefore, there is
-	   no drive letter, and the path is absolute. */
+       when the pioctl call is made, we perform an ioctl to afsrdr
+       and it returns the correct (full) path.  therefore, there is
+       no drive letter, and the path is absolute. */
     code = cm_NameI(cm_data.rootSCachep, relativePath,
                      CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                      userp, "", reqp, scpp);
 
-    if (code)
-        return code;
-
-    /* # of bytes of path */
-    code = strlen(ioctlp->inDatap) + 1;
-    ioctlp->inDatap += code;
-
-    /* This is usually nothing, but for StatMountPoint it is the file name. */
-    TranslateExtendedChars(ioctlp->inDatap);
-
-    return 0;
-#endif /* AFSIFS */
-
+    if (code) {
+	osi_Log1(afsd_logp,"cm_ParseIoctlPath code 0x%x", code);
+	return code;
+    }
+#else /* AFSIFS */
     if (relativePath[0] == relativePath[1] &&
          relativePath[1] == '\\' && 
          !_strnicmp(cm_NetbiosName,relativePath+2,strlen(cm_NetbiosName))) 
@@ -288,13 +277,18 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
                              CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, sharePath, reqp, &substRootp);
             free(sharePath);
-            if (code) 
+            if (code) {
+		osi_Log1(afsd_logp,"cm_ParseIoctlPath [1] code 0x%x", code);
                 return code;
+	    }
 
             code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, NULL, reqp, scpp);
-            if (code) 
+	    cm_ReleaseSCache(substRootp);
+            if (code) {
+		osi_Log1(afsd_logp,"cm_ParseIoctlPath [2] code 0x%x", code);
                 return code;
+	    }
         } else {
             /* otherwise, treat the name as a cellname mounted off the afs root.
              * This requires that we reconstruct the shareName string with 
@@ -316,26 +310,37 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
             code = cm_NameI(cm_data.rootSCachep, ioctlp->prefix->data,
                              CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, shareName, reqp, &substRootp);
-            if (code) 
+            if (code) {
+		osi_Log1(afsd_logp,"cm_ParseIoctlPath [3] code 0x%x", code);
                 return code;
+	    }
 
             code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                             userp, NULL, reqp, scpp);
-            if (code) 
+            if (code) {
+		cm_ReleaseSCache(substRootp);
+		osi_Log1(afsd_logp,"cm_ParseIoctlPath code [4] 0x%x", code);
                 return code;
+	    }
         }
     } else {
         code = cm_NameI(cm_data.rootSCachep, ioctlp->prefix->data,
                          CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                          userp, ioctlp->tidPathp, reqp, &substRootp);
-        if (code) 
+        if (code) {
+	    osi_Log1(afsd_logp,"cm_ParseIoctlPath [6] code 0x%x", code);
             return code;
+	}
         
         code = cm_NameI(substRootp, relativePath, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                          userp, NULL, reqp, scpp);
-        if (code) 
+        if (code) {
+	    cm_ReleaseSCache(substRootp);
+	    osi_Log1(afsd_logp,"cm_ParseIoctlPath [7] code 0x%x", code);
             return code;
+	}
     }
+#endif /* AFSIFS */
 
     /* # of bytes of path */
     code = (long)strlen(ioctlp->inDatap) + 1;
@@ -344,7 +349,11 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
     /* This is usually nothing, but for StatMountPoint it is the file name. */
     TranslateExtendedChars(ioctlp->inDatap);
 
+    if (substRootp)
+	cm_ReleaseSCache(substRootp);
+
     /* and return success */
+    osi_Log1(afsd_logp,"cm_ParseIoctlPath [8] code 0x%x", code);
     return 0;
 }
 
@@ -405,7 +414,7 @@ long cm_ParseIoctlParent(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
     long code;
     char tbuffer[1024];
     char *tp, *jp;
-    cm_scache_t *substRootp;
+    cm_scache_t *substRootp = NULL;
 
     StringCbCopyA(tbuffer, sizeof(tbuffer), ioctlp->inDatap);
     tp = strrchr(tbuffer, '\\');
@@ -460,6 +469,7 @@ long cm_ParseIoctlParent(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 
             code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, NULL, reqp, scpp);
+	    cm_ReleaseSCache(substRootp);
             if (code) return code;
         } else {
             /* otherwise, treat the name as a cellname mounted off the afs root.
@@ -485,6 +495,7 @@ long cm_ParseIoctlParent(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 
             code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                             userp, NULL, reqp, scpp);
+	    cm_ReleaseSCache(substRootp);
             if (code) return code;
         }
     } else {
@@ -495,6 +506,7 @@ long cm_ParseIoctlParent(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
 
         code = cm_NameI(substRootp, tbuffer, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                         userp, NULL, reqp, scpp);
+	cm_ReleaseSCache(substRootp);
         if (code) return code;
     }
 
@@ -1017,7 +1029,7 @@ long cm_IoctlDeleteMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
         
     /* if something went wrong, bail out now */
     if (code) {
-        goto done;
+        goto done2;
     }
         
     lock_ObtainMutex(&scp->mx);
@@ -1026,7 +1038,7 @@ long cm_IoctlDeleteMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
     if (code) {     
         lock_ReleaseMutex(&scp->mx);
         cm_ReleaseSCache(scp);
-        goto done;
+        goto done2;
     }
 
     /* now check that this is a real mount point */
@@ -1034,7 +1046,7 @@ long cm_IoctlDeleteMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
         lock_ReleaseMutex(&scp->mx);
         cm_ReleaseSCache(scp);
         code = CM_ERROR_INVAL;
-        goto done;
+        goto done1;
     }
 
     /* time to make the RPC, so drop the lock */
@@ -1048,7 +1060,12 @@ long cm_IoctlDeleteMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
                           FILE_NOTIFY_CHANGE_DIR_NAME,
                           dscp, cp, NULL, TRUE);
 
-  done:
+  done1:
+    lock_ObtainMutex(&scp->mx);
+    cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+    lock_ReleaseMutex(&scp->mx);
+
+  done2:
     cm_ReleaseSCache(dscp);
     return code;
 }
@@ -1672,6 +1689,7 @@ long cm_IoctlCreateMountPoint(struct smb_ioctl *ioctlp, struct cm_user *userp)
         osi_Log0(afsd_logp,"IoctlCreateMountPoint within Freelance root dir");
         code = cm_FreelanceAddMount(leaf, fullCell, volume, 
                                     *ioctlp->inDatap == '%', NULL);
+	cm_ReleaseSCache(dscp);
         return code;
     }
 #endif
@@ -1730,6 +1748,7 @@ long cm_IoctlSymlink(struct smb_ioctl *ioctlp, struct cm_user *userp)
         }
         osi_Log0(afsd_logp,"IoctlCreateSymlink within Freelance root dir");
         code = cm_FreelanceAddSymlink(leaf, cp, NULL);
+	cm_ReleaseSCache(dscp);
         return code;
     }
 #endif
@@ -1863,6 +1882,7 @@ long cm_IoctlDeletelink(struct smb_ioctl *ioctlp, struct cm_user *userp)
          * the freelance code to do the add. */
         osi_Log0(afsd_logp,"IoctlDeletelink from Freelance root dir");
         code = cm_FreelanceRemoveSymlink(cp);
+	cm_ReleaseSCache(dscp);
         return code;
     }
 #endif
@@ -1870,32 +1890,25 @@ long cm_IoctlDeletelink(struct smb_ioctl *ioctlp, struct cm_user *userp)
     code = cm_Lookup(dscp, cp, CM_FLAG_NOMOUNTCHASE, userp, &req, &scp);
         
     /* if something went wrong, bail out now */
-    if (code) {
-        goto done;
-    }
+    if (code)
+        goto done3;
         
     lock_ObtainMutex(&scp->mx);
     code = cm_SyncOp(scp, NULL, userp, &req, 0,
                       CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
-    if (code) {     
-        lock_ReleaseMutex(&scp->mx);
-        cm_ReleaseSCache(scp);
-        goto done;
-    }
+    if (code)
+        goto done2;
 	
     /* now check that this is a real symlink */
     if (scp->fileType != CM_SCACHETYPE_SYMLINK &&
         scp->fileType != CM_SCACHETYPE_DFSLINK &&
         scp->fileType != CM_SCACHETYPE_INVALID) {
-        lock_ReleaseMutex(&scp->mx);
-        cm_ReleaseSCache(scp);
         code = CM_ERROR_INVAL;
-        goto done;
+        goto done1;
     }
 	
     /* time to make the RPC, so drop the lock */
     lock_ReleaseMutex(&scp->mx);
-    cm_ReleaseSCache(scp);
         
     /* easier to do it this way */
     code = cm_Unlink(dscp, cp, userp, &req);
@@ -1905,7 +1918,15 @@ long cm_IoctlDeletelink(struct smb_ioctl *ioctlp, struct cm_user *userp)
                           | FILE_NOTIFY_CHANGE_DIR_NAME,
                           dscp, cp, NULL, TRUE);
 
-  done:
+    lock_ObtainMutex(&scp->mx);
+  done1:
+    cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+
+  done2:
+    lock_ReleaseMutex(&scp->mx);
+    cm_ReleaseSCache(scp);
+
+  done3:
     cm_ReleaseSCache(dscp);
     return code;
 }
@@ -2001,7 +2022,9 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
     afs_uuid_t uuid;
     int flags;
     char sessionKey[8];
+#ifndef AFSIFS
     char *smbname;
+#endif
     int release_userp = 0;
     char * wdir = NULL;
 
@@ -2048,7 +2071,7 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
         uname = tp;
         tp += strlen(tp) + 1;
 
-#ifndef AFSIFS			/* no SMB username, so we cannot log based on this */
+#ifndef AFSIFS	/* no SMB username, so we cannot logon based on this */
         if (flags & PIOCTL_LOGON) {
             /* SMB user name with which to associate tokens */
             smbname = tp;
@@ -2058,7 +2081,7 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
             tp += strlen(tp) + 1;
         } else {
             osi_Log1(smb_logp,"cm_IoctlSetToken for user [%s]",
-                     osi_LogSaveString(smb_logp,uname));
+                     osi_LogSaveString(smb_logp, uname));
         }
 #endif
 
@@ -2073,11 +2096,13 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
         osi_Log0(smb_logp,"cm_IoctlSetToken - no name specified");
     }
 
+#ifndef AFSIFS
     if (flags & PIOCTL_LOGON) {
         userp = smb_FindCMUserByName(smbname, ioctlp->fidp->vcp->rname,
 				     SMB_FLAG_CREATE|SMB_FLAG_AFSLOGON);
 	release_userp = 1;
     }
+#endif /* AFSIFS */
 
     /* store the token */
     lock_ObtainMutex(&userp->mx);
@@ -2647,6 +2672,25 @@ long cm_IoctlGetSMBName(smb_ioctl_t *ioctlp, cm_user_t *userp)
   return 0;
 }
 
+long cm_IoctlUUIDControl(struct smb_ioctl * ioctlp, struct cm_user *userp)
+{
+    long cmd;
+    afsUUID uuid;
+
+    memcpy(&cmd, ioctlp->inDatap, sizeof(long));
+
+    if (cmd) {             /* generate a new UUID */
+        UuidCreate((UUID *) &uuid);
+        cm_data.Uuid = uuid;
+	cm_ForceNewConnectionsAllServers();
+    }
+
+    memcpy(ioctlp->outDatap, &cm_data.Uuid, sizeof(cm_data.Uuid));
+    ioctlp->outDatap += sizeof(cm_data.Uuid);
+
+    return 0;
+}
+
 /* 
  * functions to dump contents of various structures. 
  * In debug build (linked with crt debug library) will dump allocated but not freed memory
