@@ -16,7 +16,11 @@
 
 #include <ntsecapi.h>
 
+/* Support largefiles by default */
+#define AFS_LARGEFILES
+
 /* basic core protocol SMB structure */
+#pragma pack(push, 1)
 typedef struct smb {
     unsigned char id[4];
     unsigned char com;
@@ -34,7 +38,7 @@ typedef struct smb {
     unsigned char wct;
     unsigned char vdata[1];
 } smb_t;
-
+#pragma pack(pop)
 
 /* reb values */
 #define SMB_FLAGS_SUPPORT_LOCKREAD         0x01
@@ -78,11 +82,33 @@ typedef struct smb {
 #define SMB_QUERY_FILE_COMPRESSION_INFO 0x10B
 #define SMB_QUERY_FILE_UNIX_BASIC       0x200
 #define SMB_QUERY_FILE_UNIX_LINK        0x201
+#define SMB_INFO_PASSTHROUGH           0x1000
 
+#define SMB_SET_FILE_BASIC_INFO         0x101
+#define SMB_SET_FILE_DISPOSITION_INFO   0x102
+#define SMB_SET_FILE_ALLOCATION_INFO    0x103
+#define SMB_SET_FILE_END_OF_FILE_INFO   0x104
 #define SMB_SET_FILE_UNIX_BASIC         0x200
 #define SMB_SET_FILE_UNIX_LINK          0x201
 #define SMB_SET_FILE_UNIX_HLINK         0x203
 
+#define SMB_INFO_ALLOCATION 		1
+#define SMB_INFO_VOLUME			2
+#define SMB_QUERY_FS_LABEL_INFO         0x101
+#define SMB_QUERY_FS_VOLUME_INFO 	0x102
+#define SMB_QUERY_FS_SIZE_INFO		0x103
+#define SMB_QUERY_FS_DEVICE_INFO	0x104
+#define SMB_QUERY_FS_ATTRIBUTE_INFO	0x105
+#define SMB_QUERY_FS_QUOTA_INFO         0x106
+#define SMB_QUERY_FS_CONTROL_INFO       0x107
+#define SMB_INFO_UNIX			0x200
+#define SMB_INFO_MACOS			0x301
+
+#define SMB_FIND_FILE_DIRECTORY_INFO      0x101
+#define SMB_FIND_FILE_FULL_DIRECTORY_INFO 0x102
+#define SMB_FIND_FILE_NAMES_INFO	  0x103
+#define SMB_FIND_FILE_BOTH_DIRECTORY_INFO 0x104
+
 /* more defines */
 #define SMB_NOPCODES		256	/* # of opcodes in the dispatch table */
 
@@ -340,7 +366,7 @@ typedef struct smb_fid {
                                            originally (used to close
                                            the file if session is
                                            terminated) */
-    long offset;			/* our file pointer */
+    osi_hyper_t offset;			/* our file pointer */
     smb_ioctl_t *ioctlp;		/* ptr to ioctl structure */
 					/* Under NT, we may need to know the
 					 * parent directory and pathname used
@@ -372,17 +398,19 @@ typedef struct smb_fid {
 #define SMB_FID_MTIMESETDONE		0x80	/* have set modtime via Tr2 */
 #define SMB_FID_LOOKSLIKECOPY	(SMB_FID_LENGTHSETDONE | SMB_FID_MTIMESETDONE)
 #define SMB_FID_NTOPEN			0x100	/* have dscp and pathp */
+#define SMB_FID_SEQUENTIAL		0x200
+#define SMB_FID_RANDOM			0x400
 
 #define SMB_FID_SHARE_READ              0x1000
 #define SMB_FID_SHARE_WRITE             0x2000
 
-#define SMB_FID_QLOCK_HIGH              0x0fe00000
+#define SMB_FID_QLOCK_HIGH              0x7f000000
 #define SMB_FID_QLOCK_LOW               0x00000000
 #define SMB_FID_QLOCK_LENGTH            1
 #define SMB_FID_QLOCK_PID               0
 
 /*
- * SMB file attributes (32-bit)
+ * SMB file attributes (16-bit)
  */
 #define SMB_ATTR_READONLY       0x0001
 #define SMB_ATTR_HIDDEN         0x0002 /* hidden file for the purpose of dir listings */
@@ -391,6 +419,8 @@ typedef struct smb_fid {
 #define SMB_ATTR_DIRECTORY      0x0010
 #define SMB_ATTR_ARCHIVE        0x0020
 #define SMB_ATTR_DEVICE         0x0040
+
+/* the following are Extended File Attributes (32-bit) */
 #define SMB_ATTR_NORMAL         0x0080 /* normal file. Only valid if used alone */
 #define SMB_ATTR_TEMPORARY      0x0100
 #define SMB_ATTR_SPARSE_FILE    0x0200 /* used with dfs links */
@@ -399,6 +429,13 @@ typedef struct smb_fid {
 #define SMB_ATTR_OFFLINE        0x1000
 #define SMB_ATTR_NOT_CONTENT_INDEXED 0x2000
 #define SMB_ATTR_ENCRYPTED      0x4000
+#define SMB_ATTR_POSIX_SEMANTICS	0x01000000
+#define SMB_ATTR_BACKUP_SEMANTICS	0x02000000
+#define SMB_ATTR_DELETE_ON_CLOSE	0x04000000
+#define SMB_ATTR_SEQUENTIAL_SCAN	0x08000000
+#define SMB_ATTR_RANDOM_ACCESS		0x10000000
+#define SMB_ATTR_NO_BUFFERING		0x20000000
+#define SMB_ATTR_WRITE_THROUGH		0x80000000
 
 #define LOCKING_ANDX_SHARED_LOCK        0x01 /* Read-only lock */
 #define LOCKING_ANDX_OPLOCK_RELEASE     0x02 /* Oplock break notification */
@@ -541,6 +578,8 @@ extern long smb_LookupTIDPath(smb_vc_t *vcp, unsigned short tid, char ** tidPath
 
 extern smb_fid_t *smb_FindFID(smb_vc_t *vcp, unsigned short fid, int flags);
 
+extern smb_fid_t *smb_FindFIDByScache(smb_vc_t *vcp, cm_scache_t * scp);
+
 extern void smb_HoldFIDNoLock(smb_fid_t *fidp);
 
 extern void smb_ReleaseFID(smb_fid_t *fidp);
@@ -570,7 +609,11 @@ extern unsigned char *smb_GetSMBData(smb_packet_t *smbp, int *nbytesp);
 
 extern void smb_SetSMBDataLength(smb_packet_t *smbp, unsigned int dsize);
 
-extern unsigned int smb_GetSMBParm(smb_packet_t *smbp, int parm);
+extern unsigned short smb_GetSMBParm(smb_packet_t *smbp, int parm);
+
+extern unsigned char smb_GetSMBParmByte(smb_packet_t *smbp, int parm);
+
+extern unsigned int smb_GetSMBParmLong(smb_packet_t *smbp, int parm);
 
 extern unsigned int smb_GetSMBOffsetParm(smb_packet_t *smbp, int parm, int offset);
 
@@ -668,10 +711,6 @@ extern unsigned int smb_Attributes(cm_scache_t *scp);
 
 extern int smb_ChainFID(int fid, smb_packet_t *inp);
 
-extern smb_fid_t *smb_FindFID(smb_vc_t *vcp, unsigned short fid, int flags);
-
-extern void smb_ReleaseFID(smb_fid_t *fidp);
-
 extern unsigned char *smb_ParseDataBlock(unsigned char *inp, char **chainpp, int *lengthp);
 
 extern unsigned char *smb_ParseASCIIBlock(unsigned char *inp, char **chainpp);
@@ -680,6 +719,14 @@ extern unsigned char *smb_ParseVblBlock(unsigned char *inp, char **chainpp, int
 
 extern int smb_SUser(cm_user_t *userp);
 
+#ifndef DJGPP
+long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count, char *op,
+	cm_user_t *userp, long *writtenp);
+#else /* DJGPP */
+long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count, char *op,
+	cm_user_t *userp, long *writtenp, int dosflag);
+#endif /* !DJGPP */
+
 #ifndef DJGPP
 extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count,
 	char *op, cm_user_t *userp, long *readp);
@@ -701,6 +748,13 @@ extern DWORD smb_ServerExceptionFilter(void);
 extern void smb_UpdateServerPriority(void);
 extern void smb_SetRequestStartTime(void);
 extern void smb_ResetServerPriority(void);
+extern void smb_RestartListeners(void);
+extern void smb_StopListeners(void);
+extern void smb_StopListener(NCB *ncbp, int lana);
+
+#define SMB_LISTENER_UNINITIALIZED -1
+#define SMB_LISTENER_STOPPED 0
+#define SMB_LISTENER_STARTED 1
 
 /* include other include files */
 #include "smb3.h"
