@@ -1,7 +1,7 @@
 /*
  * osi_sysctl.c: Linux sysctl interface to OpenAFS
  *
- * $Id: osi_sysctl.c,v 1.8 2005/04/27 12:37:56 shadow Exp $
+ * $Id: osi_sysctl.c,v 1.9.2.1 2006/11/09 23:12:51 shadow Exp $
  *
  * Written Jan 30, 2002 by Kris Van Hees (Sine Nomine Associates)
  */
@@ -14,7 +14,12 @@
 #include "afs/afs_stats.h"	/* afs statistics */
 
 #include <linux/sysctl.h>
+#ifdef CONFIG_H_EXISTS
 #include <linux/config.h>
+#endif
+
+/* From afs_util.c */
+extern afs_int32 afs_new_inum;
 
 /* From afs_analyze.c */
 extern afs_int32 hm_retry_RO;
@@ -82,6 +87,10 @@ static ctl_table afs_sysctl_table[] = {
      &afs_cacheBlocks, sizeof(afs_int32), 0644, NULL,
      &proc_dointvec}
     ,
+    {7, "md5inum",
+     &afs_new_inum, sizeof(afs_int32), 0644, NULL,
+     &proc_dointvec}
+    ,
     {0}
 };
 
