@@ -41,19 +41,22 @@ struct osi_socket {
 
 struct osi_stat {
     afs_int32 size;		/* file size in bytes */
-    afs_int32 blksize;		/* optimal transfer size in bytes */
     afs_int32 mtime;		/* modification date */
     afs_int32 atime;		/* access time */
 };
 
 struct osi_file {
     afs_int32 size;		/* file size in bytes XXX Must be first field XXX */
+#ifdef AFS_LINUX26_ENV
+    struct file *filp;		/* May need this if we really open the file. */
+#else
 #ifdef AFS_LINUX22_ENV
     struct dentry dentry;	/* merely to hold the pointer to the inode. */
     struct file file;		/* May need this if we really open the file. */
 #else
     struct vnode *vnode;
 #endif
+#endif
 #if	defined(AFS_HPUX102_ENV)
     k_off_t offset;
 #else
@@ -119,30 +122,41 @@ struct afs_osi_WaitHandle {
 /*
  * Vnode related macros
  */
-#ifndef AFS_OBSD_ENV
-#if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
-extern int (**afs_vnodeop_p) ();
-#define IsAfsVnode(vc)      ((vc)->v_op == afs_vnodeop_p)
-#define SetAfsVnode(vc)     (vc)->v_op = afs_vnodeop_p
+#if defined(AFS_DARWIN80_ENV)
+#define vType(vc)               vnode_vtype(AFSTOV(vc))
+#define vSetVfsp(vc, vfsp)      
+#define vSetType(vc, type)      (vc)->m.Type = (type)
+extern int afs_vfs_typenum;
+#define SetAfsVnode(vn)         /* nothing; done in getnewvnode() */
+#define IsAfsVnode(v) (vfs_typenum(vnode_mount((v))) == afs_vfs_typenum)
+#else
+#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV) || defined(AFS_LINUX22_ENV)
+#define vSetVfsp(vc, vfsp)      AFSTOV(vc)->v_mount = (vfsp)
+#define vSetType(vc, type)      AFSTOV(vc)->v_type = (type)
+#define vType(vc)               AFSTOV(vc)->v_type
 #else
-extern struct vnodeops *afs_ops;
-#define	IsAfsVnode(vc)	    ((vc)->v_op == afs_ops)
-#define	SetAfsVnode(vc)	    (vc)->v_op = afs_ops
-#endif
 #define	vType(vc)	    (vc)->v.v_type
 #define	vSetType(vc,type)   (vc)->v.v_type = (type)
 #define	vSetVfsp(vc,vfsp)   (vc)->v.v_vfsp = (vfsp)
 #endif
+#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+extern int (**afs_vnodeop_p) ();
+#define IsAfsVnode(v)      ((v)->v_op == afs_vnodeop_p)
+#define SetAfsVnode(v)     /* nothing; done in getnewvnode() */
+#else
+extern struct vnodeops *afs_ops;
+#define	IsAfsVnode(v)	    ((v)->v_op == afs_ops)
+#define	SetAfsVnode(v)	    (v)->v_op = afs_ops
+#endif
+#endif
 
 #ifdef AFS_SGI65_ENV
-#define	gop_lookupname(fnamep,segflg,followlink,dirvpp,compvpp) \
-             lookupname((fnamep),(segflg),(followlink),(dirvpp),(compvpp),\
+#define	gop_lookupname(fnamep,segflg,followlink,compvpp) \
+             lookupname((fnamep),(segflg),(followlink),NULL,(compvpp),\
 			NULL)
 #else
-#ifndef AFS_OBSD_ENV
-#define	gop_lookupname(fnamep,segflg,followlink,dirvpp,compvpp) \
-             lookupname((fnamep),(segflg),(followlink),(dirvpp),(compvpp))
-#endif
+#define	gop_lookupname(fnamep,segflg,followlink,compvpp) \
+             lookupname((fnamep),(segflg),(followlink),NULL,(compvpp))
 #endif
 
 /*
@@ -173,16 +187,21 @@ extern struct vnodeops *afs_ops;
   * In 64 bit HP-UX the timeval structure has a 64 bit member.
   */
 
-
-#if defined(AFS_HPUX_ENV) || (defined(AFS_SUN57_ENV) && !defined(AFS_SUN510_ENV)) || defined(AFS_LINUX_64BIT_KERNEL) || (defined(AFS_SGI61_ENV) && defined(KERNEL) && defined(_K64U64))
+#if defined(AFS_HPUX_ENV) || defined(AFS_LINUX_64BIT_KERNEL) || (defined(AFS_SGI61_ENV) && defined(KERNEL) && defined(_K64U64))
 typedef struct {
     afs_int32 tv_sec;
     afs_int32 tv_usec;
 } osi_timeval_t;
+#elif defined(AFS_SUN57_ENV)
+typedef struct timeval32 osi_timeval_t;
 #else
 typedef struct timeval osi_timeval_t;
 #endif /* AFS_SGI61_ENV */
 
+#define osi_getpid() 		getpid()
+
+#define osi_getpid() 		getpid()
+
 /*
  * osi_ThreadUnique() should yield a value that can be found in ps
  * output in order to draw correspondences between ICL traces and what
@@ -200,9 +219,7 @@ typedef struct timeval osi_timeval_t;
 
 #ifdef AFS_GLOBAL_SUNLOCK
 #define AFS_ASSERT_GLOCK() \
-    (ISAFS_GLOCK() || (osi_Panic("afs global lock not held at %s:%d\n", __FILE__, __LINE__), 0))
-#define AFS_ASSERT_RXGLOCK() \
-    (ISAFS_RXGLOCK() || (osi_Panic("rx global lock not held at %s:%d\n", __FILE__, __LINE__), 0))
+    do { if (!ISAFS_GLOCK()) osi_Panic("afs global lock not held at %s:%d\n", __FILE__, __LINE__); } while (0)
 #endif /* AFS_GLOBAL_SUNLOCK */
 
 #ifdef RX_ENABLE_LOCKS
@@ -220,10 +237,6 @@ typedef struct timeval osi_timeval_t;
 #define AFS_GUNLOCK()
 #define ISAFS_GLOCK() 1
 #define AFS_ASSERT_GLOCK()
-#define AFS_RXGLOCK()
-#define AFS_RXGUNLOCK()
-#define ISAFS_RXGLOCK() 1
-#define AFS_ASSERT_RXGLOCK()
 #endif
 
 /* On an MP that uses multithreading, splnet is not sufficient to provide
@@ -260,7 +273,7 @@ typedef struct timeval osi_timeval_t;
    calling VREF does not */
 #define AFS_FAST_HOLD(vp) osi_vnhold((vp),0)
 #else
-#define AFS_FAST_HOLD(vp) VN_HOLD(&(vp)->v)
+#define AFS_FAST_HOLD(vp) VN_HOLD(AFSTOV(vp))
 #endif
 #define AFS_FAST_RELE(vp) AFS_RELE(AFSTOV(vp))
 
@@ -269,6 +282,11 @@ typedef struct timeval osi_timeval_t;
  * and kernel space. Call these to avoid taking page faults while
  * holding the global lock.
  */
+#ifdef CAST_USER_ADDR_T
+#define __U(X) CAST_USER_ADDR_T((X))
+#else
+#define __U(X) (X)
+#endif
 #ifdef AFS_GLOBAL_SUNLOCK
 
 #define AFS_COPYIN(SRC,DST,LEN,CODE)				\
@@ -276,7 +294,7 @@ typedef struct timeval osi_timeval_t;
 	    int haveGlock = ISAFS_GLOCK();			\
 	    if (haveGlock)					\
 		AFS_GUNLOCK();					\
-	    CODE = copyin((SRC),(DST),(LEN));			\
+	    CODE = copyin(__U((SRC)),(DST),(LEN));			\
 	    if (haveGlock)					\
 		AFS_GLOCK();					\
 	} while(0)
@@ -286,7 +304,7 @@ typedef struct timeval osi_timeval_t;
 	    int haveGlock = ISAFS_GLOCK();			\
 	    if (haveGlock)					\
 		AFS_GUNLOCK();					\
-	    CODE = copyinstr((SRC),(DST),(LEN),(CNT));		\
+	    CODE = copyinstr(__U((SRC)),(DST),(LEN),(CNT));		\
 	    if (haveGlock)					\
 		AFS_GLOCK();					\
 	} while(0)
@@ -296,11 +314,23 @@ typedef struct timeval osi_timeval_t;
 	    int haveGlock = ISAFS_GLOCK();			\
 	    if (haveGlock)					\
 		AFS_GUNLOCK();					\
-	    CODE = copyout((SRC),(DST),(LEN));			\
+	    CODE = copyout((SRC),__U((DST)),(LEN));			\
 	    if (haveGlock)					\
 		AFS_GLOCK();					\
 	} while(0)
 
+#if defined(AFS_DARWIN80_ENV)
+#define AFS_UIOMOVE(SRC,LEN,RW,UIO,CODE)			\
+	do {							\
+	    int haveGlock = ISAFS_GLOCK();			\
+	    if (haveGlock)					\
+		AFS_GUNLOCK();					\
+	    uio_setrw((UIO),(RW));				\
+	    CODE = uiomove((SRC),(LEN),(UIO));			\
+	    if (haveGlock)					\
+		AFS_GLOCK();					\
+	} while(0)
+#else
 #if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 #define AFS_UIOMOVE(SRC,LEN,RW,UIO,CODE)			\
 	do {							\
@@ -323,24 +353,32 @@ typedef struct timeval osi_timeval_t;
 		AFS_GLOCK();					\
 	} while(0)
 #endif
+#endif /* AFS_DARWIN80_ENV */
 
 #else /* AFS_GLOBAL_SUNLOCK */
 
 #define AFS_COPYIN(SRC,DST,LEN,CODE)				\
 	do {							\
-	    CODE = copyin((SRC),(DST),(LEN));			\
+	    CODE = copyin(__U((SRC)),(DST),(LEN));			\
 	} while(0)
 
 #define AFS_COPYINSTR(SRC,DST,LEN,CNT,CODE)			\
 	do {							\
-	    CODE = copyinstr((SRC),(DST),(LEN),(CNT));		\
+	    CODE = copyinstr(__U((SRC)),(DST),(LEN),(CNT));		\
 	} while(0)
 
 #define AFS_COPYOUT(SRC,DST,LEN,CODE)				\
 	do {							\
-	    CODE = copyout((SRC),(DST),(LEN));			\
+	    CODE = copyout((SRC),__U((DST)),(LEN));			\
 	} while(0)
 
+#if defined(AFS_DARWIN80_ENV)
+#define AFS_UIOMOVE(SRC,LEN,RW,UIO,CODE)			\
+	do {							\
+	    uio_setrw((UIO),(RW));				\
+	    CODE = uiomove((SRC),(LEN),(UIO));			\
+	} while(0)
+#else /* AFS_OSF_ENV || AFS_FBSD_ENV */
 #if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 #define AFS_UIOMOVE(SRC,LEN,RW,UIO,CODE)			\
 	do {							\
@@ -353,9 +391,23 @@ typedef struct timeval osi_timeval_t;
 	    CODE = uiomove((SRC),(LEN),(RW),(UIO));		\
 	} while(0)
 #endif /* AFS_OSF_ENV || AFS_FBSD_ENV */
+#endif /* AFS_DARWIN80_ENV */
 
 #endif /* AFS_GLOBAL_SUNLOCK */
 
+#ifdef AFS_DARWIN80_ENV
+#define AFS_UIO_OFFSET(uio) uio_offset(uio)
+#define AFS_UIO_RESID(uio) (int)uio_resid(uio)
+#define AFS_UIO_SETOFFSET(uio, off) uio_setoffset(uio, off)
+#define AFS_UIO_SETRESID(uio, val) uio_setresid(uio, val)
+#else
+#define AFS_UIO_OFFSET(uio) (uio)->uio_offset
+#define AFS_UIO_RESID(uio) (uio)->uio_resid
+#define AFS_UIO_SETOFFSET(uio, off) (uio)->uio_offset = off
+#define AFS_UIO_SETRESID(uio, val) (uio)->uio_resid = val
+#endif
+
+
 /*
  * encapsulation of kernel data structure accesses
  */
