@@ -50,6 +50,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/ptserver/ptprocs.c,v 1.29.2.1 2006/07/04 03:57:24 shadow Exp $");
 
 #include <afs/stds.h>
 #include <ctype.h>
@@ -67,7 +69,13 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include "ptserver.h"
 #include "pterror.h"
 #include "ptprototypes.h"
@@ -213,7 +221,7 @@ SPR_INewEntry(call, aname, aid, oid)
     code = iNewEntry(call, aname, aid, oid, &cid);
     osi_auditU(call, PTS_INewEntEvent, code, AUD_ID, aid, AUD_STR, aname,
 	       AUD_ID, oid, AUD_END);
-    ViceLog(5, ("PTS_INewEntry: code %d cid %d aid %d aname %s oid %d\n", code, cid, aid, aname, oid));
+    ViceLog(25, ("PTS_INewEntry: code %d cid %d aid %d aname %s oid %d", code, cid, aid, aname, oid));
     return code;
 }
 
@@ -293,7 +301,7 @@ SPR_NewEntry(call, aname, flag, oid, aid)
     code = newEntry(call, aname, flag, oid, aid, &cid);
     osi_auditU(call, PTS_NewEntEvent, code, AUD_ID, *aid, AUD_STR, aname,
 	       AUD_ID, oid, AUD_END);
-    ViceLog(5, ("PTS_NewEntry: code %d cid %d aid %d aname %s oid %d\n", code, cid, *aid, aname, oid));
+    ViceLog(25, ("PTS_NewEntry: code %d cid %d aid %d aname %s oid %d", code, cid, *aid, aname, oid));
     return code;
 }
 
@@ -365,7 +373,7 @@ SPR_WhereIsIt(call, aid, apos)
     code = whereIsIt(call, aid, apos, &cid);
     osi_auditU(call, PTS_WheIsItEvent, code, AUD_ID, aid, AUD_LONG, *apos,
 	       AUD_END);
-    ViceLog(125, ("PTS_WhereIsIt: code %d cid %d aid %d apos %d\n", code, cid, aid, *apos));
+    ViceLog(125, ("PTS_WhereIsIt: code %d cid %d aid %d apos %d", code, cid, aid, *apos));
     return code;
 }
 
@@ -419,7 +427,7 @@ SPR_DumpEntry(call, apos, aentry)
 
     code = dumpEntry(call, apos, aentry, &cid);
     osi_auditU(call, PTS_DmpEntEvent, code, AUD_LONG, apos, AUD_END);
-    ViceLog(125, ("PTS_DumpEntry: code %d cid %d apos %d\n", code, cid, apos));
+    ViceLog(125, ("PTS_DumpEntry: code %d cid %d apos %d", code, cid, apos));
     return code;
 }
 
@@ -483,7 +491,7 @@ SPR_AddToGroup(call, aid, gid)
     code = addToGroup(call, aid, gid, &cid);
     osi_auditU(call, PTS_AdToGrpEvent, code, AUD_ID, gid, AUD_ID, aid,
 	       AUD_END);
-    ViceLog(5, ("PTS_AddToGroup: code %d cid %d gid %d aid %d\n", code, cid, gid, aid));
+    ViceLog(5, ("PTS_AddToGroup: code %d cid %d gid %d aid %d", code, cid, gid, aid));
     return code;
 }
 
@@ -576,7 +584,7 @@ SPR_NameToID(call, aname, aid)
 
     code = nameToID(call, aname, aid);
     osi_auditU(call, PTS_NmToIdEvent, code, AUD_END);
-    ViceLog(125, ("PTS_NameToID: code %d\n", code));
+    ViceLog(125, ("PTS_NameToID: code %d", code));
     return code;
 }
 
@@ -620,30 +628,13 @@ nameToID(call, aname, aid)
 	ABORT_WITH(tt, code);
 
     for (i = 0; i < aname->namelist_len; i++) {
-	char vname[256];
-	char *nameinst, *cell;
-
-	strncpy(vname, aname->namelist_val[i], sizeof(vname));
-	vname[sizeof(vname)-1] ='\0';
-
-	nameinst = vname;
-	cell = strchr(vname, '@');
-	if (cell) {
-	    *cell = '\0';
-	    cell++;
-	}
-
-	if (cell && afs_is_foreign_ticket_name(nameinst,NULL,cell,pr_realmName))
-	    code = NameToID(tt, aname->namelist_val[i], &aid->idlist_val[i]);
-	else 
-	    code = NameToID(tt, nameinst, &aid->idlist_val[i]);
-
+	code = NameToID(tt, aname->namelist_val[i], &aid->idlist_val[i]);
 	if (code != PRSUCCESS)
 	    aid->idlist_val[i] = ANONYMOUSID;
         osi_audit(PTS_NmToIdEvent, code, AUD_STR,
 		   aname->namelist_val[i], AUD_ID, aid->idlist_val[i], 
 		   AUD_END);
-	ViceLog(125, ("PTS_NameToID: code %d aname %s aid %d\n", code,
+	ViceLog(125, ("PTS_NameToID: code %d aname %s aid %d", code,
 		      aname->namelist_val[i], aid->idlist_val[i]));
 	if (count++ > 50)
 	    IOMGR_Poll(), count = 0;
@@ -671,7 +662,7 @@ SPR_IDToName(call, aid, aname)
 
     code = idToName(call, aid, aname);
     osi_auditU(call, PTS_IdToNmEvent, code, AUD_END);
-    ViceLog(125, ("PTS_IDToName: code %d\n", code));
+    ViceLog(125, ("PTS_IDToName: code %d", code));
     return code;
 }
 
@@ -691,7 +682,7 @@ idToName(call, aid, aname)
     size = aid->idlist_len;
     if (size == 0)
 	return 0;
-    if (size < 0 || size > INT_MAX / PR_MAXNAMELEN)
+    if (size < 0)
 	return PRTOOMANY;
     aname->namelist_val = (prname *) malloc(size * PR_MAXNAMELEN);
     aname->namelist_len = 0;
@@ -721,7 +712,7 @@ idToName(call, aid, aname)
 	    sprintf(aname->namelist_val[i], "%d", aid->idlist_val[i]);
         osi_audit(PTS_IdToNmEvent, code, AUD_ID, aid->idlist_val[i],
 		  AUD_STR, aname->namelist_val[i], AUD_END);
-	ViceLog(125, ("PTS_idToName: code %d aid %d aname %s\n", code,
+	ViceLog(125, ("PTS_idToName: code %d aid %d aname %s", code,
 		      aid->idlist_val[i], aname->namelist_val[i]));
 	if (count++ > 50)
 	    IOMGR_Poll(), count = 0;
@@ -744,7 +735,7 @@ SPR_Delete(call, aid)
 
     code = Delete(call, aid, &cid);
     osi_auditU(call, PTS_DelEvent, code, AUD_ID, aid, AUD_END);
-    ViceLog(5, ("PTS_Delete: code %d cid %d aid %d\n", code, cid, aid));
+    ViceLog(25, ("PTS_Delete: code %d cid %d aid %d", code, cid, aid));
     return code;
 }
 
@@ -986,7 +977,7 @@ SPR_UpdateEntry(call, aid, name, uentry)
 
     code = UpdateEntry(call, aid, name, uentry, &cid);
     osi_auditU(call, PTS_UpdEntEvent, code, AUD_ID, aid, AUD_STR, name, AUD_END);
-    ViceLog(5, ("PTS_UpdateEntry: code %d cid %d aid %d name %s\n", code, cid, aid, name));
+    ViceLog(5, ("PTS_UpdateEntry: code %d cid %d aid %d name %s", code, cid, aid, name));
     return code;
 }
 
@@ -1084,7 +1075,7 @@ SPR_RemoveFromGroup(call, aid, gid)
     code = removeFromGroup(call, aid, gid, &cid);
     osi_auditU(call, PTS_RmFmGrpEvent, code, AUD_ID, gid, AUD_ID, aid,
 	       AUD_END);
-    ViceLog(5, ("PTS_RemoveFromGroup: code %d cid %d gid %d aid %d\n", code, cid, gid, aid));
+    ViceLog(5, ("PTS_RemoveFromGroup: code %d cid %d gid %d aid %d", code, cid, gid, aid));
     return code;
 }
 
@@ -1173,7 +1164,7 @@ SPR_GetCPS(call, aid, alist, over)
 
     code = getCPS(call, aid, alist, over, &cid);
     osi_auditU(call, PTS_GetCPSEvent, code, AUD_ID, aid, AUD_END);
-    ViceLog(125, ("PTS_GetCPS: code %d cid %d aid %d\n", code, cid, aid));
+    ViceLog(125, ("PTS_GetCPS: code %d cid %d aid %d", code, cid, aid));
     return code;
 }
 
@@ -1258,7 +1249,7 @@ SPR_GetCPS2(call, aid, ahost, alist, over)
     code = getCPS2(call, aid, ahost, alist, over, &cid);
     osi_auditU(call, PTS_GetCPS2Event, code, AUD_ID, aid, AUD_HOST, ahost,
 	       AUD_END);
-    ViceLog(125, ("PTS_GetCPS2: code %d cid %d aid %d ahost %d\n", code, cid, aid, ahost));
+    ViceLog(125, ("PTS_GetCPS2: code %d cid %d aid %d ahost %d", code, cid, aid, ahost));
     return code;
 }
 
@@ -1353,7 +1344,7 @@ SPR_GetHostCPS(call, ahost, alist, over)
 
     code = getHostCPS(call, ahost, alist, over);
     osi_auditU(call, PTS_GetHCPSEvent, code, AUD_HOST, ahost, AUD_END);
-    ViceLog(125, ("PTS_GetHostCPS: code %d ahost %d\n", code, ahost));
+    ViceLog(125, ("PTS_GetHostCPS: code %d ahost %d", code, ahost));
     return code;
 }
 
@@ -1426,7 +1417,7 @@ SPR_ListMax(call, uid, gid)
 
     code = listMax(call, uid, gid);
     osi_auditU(call, PTS_LstMaxEvent, code, AUD_END);
-    ViceLog(125, ("PTS_ListMax: code %d\n", code));
+    ViceLog(125, ("PTS_ListMax: code %d", code));
     return code;
 }
 
@@ -1474,7 +1465,7 @@ SPR_SetMax(call, aid, gflag)
     code = setMax(call, aid, gflag, &cid);
     osi_auditU(call, PTS_SetMaxEvent, code, AUD_ID, aid, AUD_LONG, gflag,
 	       AUD_END);
-    ViceLog(125, ("PTS_SetMax: code %d cid %d aid %d gflag %d\n", code, cid, aid, gflag));
+    ViceLog(125, ("PTS_SetMax: code %d cid %d aid %d gflag %d", code, cid, aid, gflag));
     return code;
 }
 
@@ -1530,7 +1521,7 @@ SPR_ListEntry(call, aid, aentry)
 
     code = listEntry(call, aid, aentry, &cid);
     osi_auditU(call, PTS_LstEntEvent, code, AUD_ID, aid, AUD_END);
-    ViceLog(125, ("PTS_ListEntry: code %d cid %d aid %d\n", code, cid, aid));
+    ViceLog(125, ("PTS_ListEntry: code %d cid %d aid %d", code, cid, aid));
     return code;
 }
 
@@ -1605,7 +1596,7 @@ SPR_ListEntries(call, flag, startindex, bulkentries, nextstartindex)
 
     code = listEntries(call, flag, startindex, bulkentries, nextstartindex, &cid);
     osi_auditU(call, PTS_LstEntsEvent, code, AUD_LONG, flag, AUD_END);
-    ViceLog(125, ("PTS_ListEntries: code %d cid %d flag %d\n", code, cid, flag));
+    ViceLog(125, ("PTS_ListEntries: code %d cid %d flag %d", code, cid, flag));
     return code;
 }
 
@@ -1751,7 +1742,7 @@ SPR_ChangeEntry(call, aid, name, oid, newid)
     code = changeEntry(call, aid, name, oid, newid, &cid);
     osi_auditU(call, PTS_ChgEntEvent, code, AUD_ID, aid, AUD_STR, name,
 	       AUD_LONG, oid, AUD_LONG, newid, AUD_END);
-    ViceLog(5, ("PTS_ChangeEntry: code %d cid %d aid %d name %s oid %d newid %d\n", code, cid, aid, name, oid, newid));
+    ViceLog(5, ("PTS_ChangeEntry: code %d cid %d aid %d name %s oid %d newid %d", code, cid, aid, name, oid, newid));
     return code;
 }
 
@@ -1820,7 +1811,7 @@ SPR_SetFieldsEntry(call, id, mask, flags, ngroups, nusers, spare1, spare2)
 	setFieldsEntry(call, id, mask, flags, ngroups, nusers, spare1,
 		       spare2, &cid);
     osi_auditU(call, PTS_SetFldEntEvent, code, AUD_ID, id, AUD_END);
-    ViceLog(5, ("PTS_SetFieldsEntry: code %d cid %d id %d\n", code, cid, id));
+    ViceLog(5, ("PTS_SetFieldsEntry: code %d cid %d id %d", code, cid, id));
     return code;
 }
 
@@ -1922,7 +1913,7 @@ SPR_ListElements(call, aid, alist, over)
 
     code = listElements(call, aid, alist, over, &cid);
     osi_auditU(call, PTS_LstEleEvent, code, AUD_ID, aid, AUD_END);
-    ViceLog(125, ("PTS_ListElements: code %d cid %d aid %d\n", code, cid, aid));
+    ViceLog(125, ("PTS_ListElements: code %d cid %d aid %d", code, cid, aid));
     return code;
 }
 
@@ -1991,7 +1982,7 @@ SPR_ListSuperGroups(call, aid, alist, over)
 
     code = listSuperGroups(call, aid, alist, over, &cid);
     osi_auditU(call, PTS_LstSGrps, code, AUD_ID, aid, AUD_END);
-    ViceLog(125, ("PTS_ListSuperGroups: code %d cid %d aid %d\n", code, cid, aid));
+    ViceLog(125, ("PTS_ListSuperGroups: code %d cid %d aid %d", code, cid, aid));
     return code;
 #else
     return RXGEN_OPCODE;
@@ -2071,7 +2062,7 @@ SPR_ListOwned(call, aid, alist, lastP)
 
     code = listOwned(call, aid, alist, lastP, &cid);
     osi_auditU(call, PTS_LstOwnEvent, code, AUD_ID, aid, AUD_END);
-    ViceLog(125, ("PTS_ListOwned: code %d cid %d aid %d\n", code, cid, aid));
+    ViceLog(125, ("PTS_ListOwned: code %d cid %d aid %d", code, cid, aid));
     return code;
 }
 
@@ -2164,7 +2155,7 @@ SPR_IsAMemberOf(call, uid, gid, flag)
     code = isAMemberOf(call, uid, gid, flag, &cid);
     osi_auditU(call, PTS_IsMemOfEvent, code, AUD_LONG, uid, AUD_LONG, gid,
 	       AUD_END);
-    ViceLog(125, ("PTS_IsAMemberOf: code %d cid %d uid %d gid %d\n", code, cid, uid, gid));
+    ViceLog(125, ("PTS_IsAMemberOf: code %d cid %d uid %d gid %d", code, cid, uid, gid));
     return code;
 }
 
