@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/dumpstuff.c,v 1.29.2.1 2006/06/20 21:40:12 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/dumpstuff.c,v 1.29.2.6 2006/12/19 04:50:46 shadow Exp $");
 
 #include <sys/types.h>
 #include <ctype.h>
@@ -535,11 +535,11 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
     afs_sfsize_t size;
 #ifdef	AFS_AIX_ENV
 #include <sys/statfs.h>
-#ifdef AFS_LARGEFILE_ENV
+#if defined(AFS_AIX52_ENV) && defined(AFS_LARGEFILE_ENV)
     struct statfs64 tstatfs;
-#else /* !AFS_LARGEFILE_ENV */
+#else /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
     struct statfs tstatfs;
-#endif /* !AFS_LARGEFILE_ENV */
+#endif /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
     int statfs_code;
 #endif
 
@@ -555,16 +555,16 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
     /* Unfortunately in AIX valuable fields such as st_blksize are 
      * gone from the stat structure.
      */
-#ifdef AFS_LARGEFILE_ENV
+#if defined(AFS_AIX52_ENV) && defined(AFS_LARGEFILE_ENV)
     statfs_code = fstatfs64(handleP->fd_fd, &tstatfs);
-#else /* !AFS_LARGEFILE_ENV */
+#else /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
     statfs_code = fstatfs(handleP->fd_fd, &tstatfs);
-#endif /* !AFS_LARGEFILE_ENV */
+#endif /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
     if (statfs_code != 0) {
         Log("DumpFile: fstatfs returned error code %d on descriptor %d\n", errno, handleP->fd_fd);
 	return VOLSERDUMPERROR;
     }
-    howMany = tstatfs.f_bsize;
+    howMany = (afs_sfsize_t) tstatfs.f_bsize;
     Log("DumpFile: fstatfs returned block size of %lld; howMany=%lld", tstatfs.f_bsize, howMany);
 #else
     howMany = status.st_blksize;
@@ -998,7 +998,7 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 	    if (Buf == NULL) {
 		STREAM_CLOSE(afile);
 		FDH_CLOSE(fdP);
-		return 1;
+		return -1;
 	    }
 	    memset((char *)Buf, 0, nVnodes * sizeof(afs_int32));
 	    STREAM_SEEK(afile, offset = vcp->diskSize, 0);
@@ -1035,12 +1035,15 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
     struct iod iod;
     register struct iod *iodp = &iod;
     afs_int32 *b1 = NULL, *b2 = NULL;
-    int s1 = 0, s2 = 0, delo = incremental, tdelo;
+    int s1 = 0, s2 = 0, delo = 0, tdelo;
     int tag;
 
     iod_Init(iodp, call);
 
     vp = avp;
+    if (V_parentId(vp) == V_id(vp))
+      delo = incremental;
+
     if (!ReadDumpHeader(iodp, &header)) {
 	Log("1 Volser: RestoreVolume: Error reading header file for dump; aborted\n");
 	return VOLSERREAD_DUMPERROR;
@@ -1056,13 +1059,10 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
 	delo = ProcessIndex(vp, vLarge, &b1, &s1, 0);
     if (!delo)
 	delo = ProcessIndex(vp, vSmall, &b2, &s2, 0);
-    if (delo) {
-	if (b1)
-	    free((char *)b1);
-	if (b2)
-	    free((char *)b2);
-	b1 = b2 = NULL;
-	s1 = s2 = 0;
+    if (delo < 0) {
+	Log("1 Volser: RestoreVolume: ProcessIndex failed; not restored\n");
+	error = VOLSERREAD_DUMPERROR;
+	goto out;
     }
 
     strncpy(vol.name, cookie->name, VOLSER_OLDMAXVOLNAME);
@@ -1070,7 +1070,6 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
     vol.cloneId = cookie->clone;
     vol.parentId = cookie->parent;
 
-
     tdelo = delo;
     while (1) {
 	int temprc;
@@ -1109,8 +1108,8 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
     if (!delo) {
 	delo = ProcessIndex(vp, vLarge, &b1, &s1, 1);
 	if (!delo)
-	    ProcessIndex(vp, vSmall, &b2, &s2, 1);
-	if (delo) {
+	    delo = ProcessIndex(vp, vSmall, &b2, &s2, 1);
+	if (delo < 0) {
 	    error = VOLSERREAD_DUMPERROR;
 	    goto clean;
 	}
@@ -1322,6 +1321,7 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 		 Error * status)
 {
     afs_int32 code;
+    afs_sfsize_t lcode;
     afs_fsize_t filesize;
     afs_fsize_t written = 0;
     register afs_uint32 size = 8192;
@@ -1367,11 +1367,11 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 	    *status = 3;
 	    break;
 	}
-	code = FDH_WRITE(handleP, p, size);
-	if (code > 0)
-	    written += code;
-	if (code != size) {
-	    Log("1 Volser: WriteFile: Error creating file in volume; restore aborted\n");
+	lcode = FDH_WRITE(handleP, p, size);
+	if (lcode > 0)
+	    written += lcode;
+	if (lcode != size) {
+	    Log("1 Volser: WriteFile: Error writing (%d,%u) bytes to vnode %d; restore aborted\n", (int)(lcode>>32), (int)(lcode & 0xffffffff), vn);
 	    *status = 4;
 	    break;
 	}
