@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/volprocs.c,v 1.32 2003/12/07 22:49:44 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/volser/volprocs.c,v 1.42.2.2 2006/12/19 03:42:21 jaltman Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -61,6 +61,7 @@ RCSID
 #include <afs/volume.h>
 #include <afs/partition.h>
 #include "vol.h"
+#include <afs/daemon_com.h>
 #include <afs/fssync.h>
 #include <afs/acl.h>
 #include "afs/audit.h"
@@ -75,6 +76,9 @@ extern int DoLogging;
 extern struct volser_trans *FindTrans(), *NewTrans(), *TransList();
 extern struct afsconf_dir *tdir;
 
+/* Needed by Irix. Leave, or include a header */
+extern char *volutil_PartitionName();
+
 extern void LogError(afs_int32 errcode);
 
 /* Forward declarations */
@@ -422,7 +426,7 @@ VolCreateVolume(struct rx_call *acid, afs_int32 apart, char *aname,
     if (error) {
 	Log("1 Volser: CreateVolume: Unable to create the volume; aborted, error code %u\n", error);
 	LogError(error);
-	DeleteTrans(tt);
+	DeleteTrans(tt, 1);
 	return EIO;
     }
     V_uniquifier(vp) = 1;
@@ -439,7 +443,7 @@ VolCreateVolume(struct rx_call *acid, afs_int32 apart, char *aname,
     if (error) {
 	Log("1 Volser: create UpdateVolume failed, code %d\n", error);
 	LogError(error);
-	DeleteTrans(tt);
+	DeleteTrans(tt, 1);
 	VDetachVolume(&junk, vp);	/* rather return the real error code */
 	return error;
     }
@@ -674,7 +678,7 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_int32 purgeId,
 	error = VOLSERTRELE_ERROR;
 	goto fail;
     }
-    DeleteTrans(ttc);
+    DeleteTrans(ttc, 1);
     return 0;
 
   fail:
@@ -687,7 +691,7 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_int32 purgeId,
 	TRELE(tt);
     }
     if (ttc)
-	DeleteTrans(ttc);
+	DeleteTrans(ttc, 1);
     return error;
 }
 
@@ -837,11 +841,11 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
 	goto fail;
     }
 
-    DeleteTrans(ttc);
+    DeleteTrans(ttc, 1);
 
     {
 	struct DiskPartition *tpartp = originalvp->partition;
-	FSYNC_askfs(cloneId, tpartp->name, FSYNC_RESTOREVOLUME, 0);
+	FSYNC_VolOp(cloneId, tpartp->name, FSYNC_VOL_BREAKCBKS, 0, NULL);
     }
     return 0;
 
@@ -853,7 +857,7 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
 	TRELE(tt);
     }
     if (ttc)
-	DeleteTrans(ttc);
+	DeleteTrans(ttc, 1);
     return error;
 }
 
@@ -910,7 +914,7 @@ VolTransCreate(struct rx_call *acid, afs_int32 volume, afs_int32 partition,
 	/* give up */
 	if (tv)
 	    VDetachVolume(&code, tv);
-	DeleteTrans(tt);
+	DeleteTrans(tt, 1);
 	return error;
     }
     tt->volume = tv;
@@ -1193,7 +1197,7 @@ SAFSVolForwardMultiple(struct rx_call *acid, afs_int32 fromTrans, afs_int32
     vp = tt->volume;
     strcpy(tt->lastProcName, "ForwardMulti");
 
-    /* (fromDate == 0) ==> incremental dump */
+    /* (fromDate == 0) ==> full dump */
     is_incremental = (fromDate ? 1 : 0);
 
     i = results->manyResults_len = destinations->manyDests_len;
@@ -1352,8 +1356,7 @@ VolRestore(struct rx_call *acid, afs_int32 atrans, afs_int32 aflags,
     DFlushVolume(V_parentId(tt->volume)); /* Ensure dir buffers get dropped */
 
     code = RestoreVolume(acid, tt->volume, (aflags & 1), cookie);	/* last is incrementalp */
-    FSYNC_askfs(tt->volid, NULL, FSYNC_RESTOREVOLUME, 0l);	/*break call backs on the
-								 * restored volume */
+    FSYNC_VolOp(tt->volid, NULL, FSYNC_VOL_BREAKCBKS, 0l, NULL);
     tt->rxCallPtr = (struct rx_call *)0;
     tcode = TRELE(tt);
 
@@ -1384,7 +1387,7 @@ VolEndTrans(struct rx_call *acid, afs_int32 destTrans, afs_int32 *rcode)
 	return ENOENT;
     }
     *rcode = tt->returnCode;
-    DeleteTrans(tt);		/* this does an implicit TRELE */
+    DeleteTrans(tt, 1);		/* this does an implicit TRELE */
 
     return 0;
 }
@@ -1419,7 +1422,7 @@ VolSetForwarding(struct rx_call *acid, afs_int32 atid, afs_int32 anewsite)
     }
     strcpy(tt->lastProcName, "SetForwarding");
     tt->rxCallPtr = acid;
-    FSYNC_askfs(tt->volid, NULL, FSYNC_MOVEVOLUME, anewsite);
+    FSYNC_VolOp(tt->volid, NULL, FSYNC_VOL_MOVE, anewsite, NULL);
     tt->rxCallPtr = (struct rx_call *)0;
     if (TRELE(tt))
 	return VOLSERTRELE_ERROR;
@@ -1537,6 +1540,12 @@ VolSetInfo(struct rx_call *acid, afs_int32 atrans,
 	td->maxquota = astatus->maxquota;
     if (astatus->dayUse != -1)
 	td->dayUse = astatus->dayUse;
+    if (astatus->creationDate != -1)
+	td->creationDate = astatus->creationDate;
+    if (astatus->updateDate != -1)
+	td->updateDate = astatus->updateDate;
+    if (astatus->spare2 != -1)
+	td->volUpdateCounter = (unsigned int)astatus->spare2;
     VUpdateVolume(&error, tv);
     tt->rxCallPtr = (struct rx_call *)0;
     if (TRELE(tt))
@@ -1663,6 +1672,9 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 
     /* Only report attached partitions */
     for (i = 0; i < VOLMAXPARTS; i++) {
+#ifdef AFS_DEMAND_ATTACH_FS
+	dp = VGetPartitionById(i, 0);
+#else
 	if (i < 26) {
 	    namehead[6] = i + 'a';
 	    namehead[7] = '\0';
@@ -1673,6 +1685,7 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 	    namehead[8] = '\0';
 	}
 	dp = VGetPartition(namehead, 0);
+#endif
 	if (dp)
 	    partList.partId[j++] = i;
     }
@@ -1783,7 +1796,7 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid, afs_int32
 		pntr->volid = volid;
 		goto drop;
 	    }
-	    tv = VAttachVolumeByName(&error, pname, volname, V_READONLY);
+	    tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
 	    if (error) {
 		pntr->status = 0;	/*things are messed up */
 		strcpy(pntr->name, volname);
@@ -1842,7 +1855,8 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid, afs_int32
 		(long)tv->header->diskstuff.weekUse[4] +
 		(long)tv->header->diskstuff.weekUse[5] +
 		(long)tv->header->diskstuff.weekUse[6];
-	    pntr->flags = pntr->spare2 = pntr->spare3 = (long)0;
+	    pntr->spare2 = V_volUpCounter(tv);
+	    pntr->flags = pntr->spare3 = (long)0;
 	    VDetachVolume(&error, tv);	/*free the volume */
 	    tv = (Volume *) 0;
 	    if (error) {
@@ -1861,7 +1875,7 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid, afs_int32
 	tv = (Volume *) 0;
     }
     if (ttc) {
-	DeleteTrans(ttc);
+	DeleteTrans(ttc, 1);
 	ttc = (struct volser_trans *)0;
     }
 
@@ -1997,7 +2011,7 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	    /*
 	     * Attach the volume, give up on the volume if we can't.
 	     */
-	    tv = VAttachVolumeByName(&error, pname, volname, V_READONLY);
+	    tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
 	    if (error) {
 		xInfoP->status = 0;	/*things are messed up */
 		strcpy(xInfoP->name, volname);
@@ -2092,7 +2106,7 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	tv = (Volume *) 0;
     }
     if (ttc) {
-	DeleteTrans(ttc);
+	DeleteTrans(ttc, 1);
 	ttc = (struct volser_trans *)0;
     }
 
@@ -2164,7 +2178,7 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 		pntr->volid = volid;
 		goto drop;
 	    }
-	    tv = VAttachVolumeByName(&error, pname, volname, V_READONLY);
+	    tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
 	    if (error) {
 		pntr->status = 0;	/*things are messed up */
 		strcpy(pntr->name, volname);
@@ -2217,7 +2231,8 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 		(long)tv->header->diskstuff.weekUse[4] +
 		(long)tv->header->diskstuff.weekUse[5] +
 		(long)tv->header->diskstuff.weekUse[6];
-	    pntr->flags = pntr->spare2 = pntr->spare3 = (long)0;
+	    pntr->spare2 = V_volUpCounter(tv);
+	    pntr->flags = pntr->spare3 = (long)0;
 	    VDetachVolume(&error, tv);	/*free the volume */
 	    tv = (Volume *) 0;
 	    if (error) {
@@ -2234,7 +2249,7 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 
       drop:
 	if (ttc) {
-	    DeleteTrans(ttc);
+	    DeleteTrans(ttc, 1);
 	    ttc = (struct volser_trans *)0;
 	}
 	pntr++;
@@ -2251,7 +2266,7 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 		    tv = (Volume *) 0;
 		}
 		if (ttc) {
-		    DeleteTrans(ttc);
+		    DeleteTrans(ttc, 1);
 		    ttc = (struct volser_trans *)0;
 		}
 		closedir(dirp);
@@ -2269,7 +2284,7 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 	    tv = (Volume *) 0;
 	}
 	if (ttc) {
-	    DeleteTrans(ttc);
+	    DeleteTrans(ttc, 1);
 	    ttc = (struct volser_trans *)0;
 	}
 	GetNextVol(dirp, volname, &volid);
@@ -2277,7 +2292,7 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
     }
     closedir(dirp);
     if (ttc)
-	DeleteTrans(ttc);
+	DeleteTrans(ttc, 1);
 
     return 0;
 }
@@ -2406,7 +2421,7 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	    /*
 	     * Attach the volume, give up on this volume if we can't.
 	     */
-	    tv = VAttachVolumeByName(&error, pname, volname, V_READONLY);
+	    tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
 	    if (error) {
 		xInfoP->status = 0;	/*things are messed up */
 		strcpy(xInfoP->name, volname);
@@ -2488,7 +2503,7 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	 * Drop the transaction we have for this volume.
 	 */
 	if (ttc) {
-	    DeleteTrans(ttc);
+	    DeleteTrans(ttc, 1);
 	    ttc = (struct volser_trans *)0;
 	}
 
@@ -2515,7 +2530,7 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 		    tv = (Volume *) 0;
 		}
 		if (ttc) {
-		    DeleteTrans(ttc);
+		    DeleteTrans(ttc, 1);
 		    ttc = (struct volser_trans *)0;
 		}
 		closedir(dirp);
@@ -2543,7 +2558,7 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	    tv = (Volume *) 0;
 	}
 	if (ttc) {
-	    DeleteTrans(ttc);
+	    DeleteTrans(ttc, 1);
 	    ttc = (struct volser_trans *)0;
 	}
 	GetNextVol(dirp, volname, &volid);
@@ -2555,7 +2570,7 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
      */
     closedir(dirp);
     if (ttc)
-	DeleteTrans(ttc);
+	DeleteTrans(ttc, 1);
     return (0);
 
 }				/*SAFSVolXListVolumes */
@@ -2633,7 +2648,7 @@ SAFSVolSetIdsTypes(struct rx_call *acid, afs_int32 atid, char name[], afs_int32
 
     code = VolSetIdsTypes(acid, atid, name, type, pId, cloneId, backupId);
     osi_auditU(acid, VS_SetIdTyEvent, code, AUD_LONG, atid, AUD_STR, name,
-	       AUD_STR, type, AUD_LONG, pId, AUD_LONG, cloneId, AUD_LONG,
+	       AUD_LONG, type, AUD_LONG, pId, AUD_LONG, cloneId, AUD_LONG,
 	       backupId, AUD_END);
     return code;
 }
@@ -2808,7 +2823,7 @@ SAFSVolConvertROtoRWvolume(struct rx_call *acid, afs_int32 partId,
 	return EIO;
     }
     close(fd);
-    FSYNC_askfs(volumeId, pname, FSYNC_RESTOREVOLUME, 0);
+    FSYNC_VolOp(volumeId, pname, FSYNC_VOL_BREAKCBKS, 0, NULL);
 
     for (dp = DiskPartitionList; dp && strcmp(dp->name, pname);
 	 dp = dp->next);
@@ -2843,8 +2858,8 @@ SAFSVolConvertROtoRWvolume(struct rx_call *acid, afs_int32 partId,
     if (unlink(opath) < 0) {
 	Log("1 SAFS_VolConvertROtoRWvolume: Couldn't unlink RO header, error = %d\n", error);
     }
-    FSYNC_askfs(volumeId, pname, FSYNC_DONE, 0);
-    FSYNC_askfs(h.id, pname, FSYNC_ON, 0);
+    FSYNC_VolOp(volumeId, pname, FSYNC_VOL_DONE, 0, NULL);
+    FSYNC_VolOp(h.id, pname, FSYNC_VOL_ON, 0, NULL);
     return 0;
 #else /* AFS_NAMEI_ENV */
     return EINVAL;
